/**
 * This class is used to build an Xml Node for the story boundaries task output.
 * Each object of this class contains the result for a video result of one specific Run
 * and only for the story boundary Task
 */

import java.io.*;
import org.w3c.dom.*;
import java.util.*;
import com.sun.xml.tree.*;

public class DistributionTree
{
    Document document = new XmlDocument();
    VideoSubmission videosub;
    TruthData truth;
    Vector resultDistribution = new Vector();
    Element video; // Xml Element, this element will be addes to the xml file result in StoriesSegmentEvalutionClass
    String fileName;
    TimeBase clippingStart, clippingEnd, length;
    int nbBoundDet, nbBoundRef, nbBoundSub,nbBoundDetSub,nbBoundRefTotal;
    float recall,precision;
      
    /**
     * Constructor method.
     * @param vid VideoSubmission  
     * @param truth truthData associated to vid
     * @param doc Document, the documnet to wich these information are going to be added
     */
    DistributionTree(VideoSubmission vid, TruthData truth, Document doc)
    {
	nbBoundDet = 0;
	nbBoundRef = 0;
	nbBoundSub = 0;
	nbBoundDetSub =0;
	nbBoundRefTotal =0;

	this.videosub = vid;
	this.truth = truth;
	this.document=doc;
	
        // Get all the results for this video
	resultDistribution = videosub.returnResultBoundaryDistribution();
	fileName = videosub.getVideoFileName();
	//clippingStart = (TimeBase) videosub.searchClippingPoints().get(1);
	//clippingEnd = (TimeBase) videosub.searchClippingPoints().get(2);
	length = videosub.getMPEG1Length();
	
	//System.out.println("\nBefore getNumber of Bound");
	//videosub.displaySegment();
	nbBoundDet = videosub.getNumbersOfReferenceBoundariesDetectedOneFile(5);
	nbBoundRef = truth.getNumberOfEvaluatedBoundaries();
	//nbBoundRefTotal = nbTotalBoundTruth;
	nbBoundSub = videosub.getNumberOfEvaluatedBoundaries();	
	nbBoundDetSub = videosub.getNbDetectedBoundaries(5);
	recall = (float)nbBoundDet / (float)nbBoundRef;
	precision = (float) nbBoundDetSub / (float)nbBoundSub;

	createTree();
    }

    /**
     * returns the complete Tree containing the results of the story boundary Task for the VideoSubmission
     * @return Element, Xml Element
     */
    public Element getTree()
    {
	return video;
    }
    
    /**
     * This method creates the tree, set the attributes and the elements
     * @return void
     */
    public void createTree()
    {
	video   = document.createElement("video");
	video.setAttribute("src",fileName);
	video.setAttribute("length",length.toString());
	//video.setAttribute("clippingStart",clippingStart.toString());
	//video.setAttribute("clippingEnd",clippingEnd.toString());
	video.setAttribute("Offset",videosub.getAlign().toString());
	video.setAttribute("NumberBoundariesEvaluatedInSubmission",Integer.toString(nbBoundSub));
	video.setAttribute("NumberBoundariesEvaluatedInTruth",Integer.toString(nbBoundRef));
	//video.setAttribute("NumberBoundariesEvaluatedInAllTruth",Integer.toString(nbBoundRefTotal));
	video.setAttribute("NumberBoundariesFoundInSub",Integer.toString(nbBoundDetSub));
	video.setAttribute("NumberBoundariesFoundInTruth",Integer.toString(nbBoundDet));
	video.setAttribute("Recall",Float.toString(recall));
	video.setAttribute("Precision",Float.toString(precision));

	Element match = createMatch();
	Element insertion = createInsertion();
	Element deletion = createDeletion();
	Element clippedRef = createClippedRef();
	Element clippedSub = createClippedSub();
	Element reference = createReference();
	Element referenceAfter = createReferenceAfterClipping();
	Element submission = createSubmission();
	Element submissionAfter = createSubmissionAfterClipping();
	
	//Uncomment theses two following lines to get the Submission and the Reference in the Result file
// 	video.appendChild(reference);
// 	video.appendChild(referenceAfter);
// 	video.appendChild(submission);
// 	video.appendChild(submissionAfter);

	video.appendChild(clippedSub);
	video.appendChild(clippedRef);
	video.appendChild(match);
	video.appendChild(insertion);
	video.appendChild(deletion);
    }
    
 
    /**
     * Create the subTree <match>. 
     * @return Element
     */
    public Element createMatch()
    {
	Element match = document.createElement("Match");
	Vector matchv = (Vector) resultDistribution.get(2);
	
	for(Enumeration e=matchv.elements();e.hasMoreElements();)
	    {
		Vector story = (Vector) e.nextElement();
		Element bound = document.createElement("Boundary");
		bound.setAttribute("TimeD",(String)story.get(0));
		//bound.setAttribute("Type",(String)story.get(1));
		match.appendChild(bound);
	    }
	return match;
    }

    /**
     * Create the subTree <insertion>. 
     * @return Element
     */
    public Element createInsertion()
    {
	Element insertion = document.createElement("Insertion");
	Vector insertionv = (Vector) resultDistribution.get(0);
	
	for(Enumeration e=insertionv.elements();e.hasMoreElements();)
	    {
		Vector story = (Vector) e.nextElement();
		Element bound = document.createElement("Boudary");
		bound.setAttribute("TimeD",(String)story.get(0));
		//bound.setAttribute("Type",(String)story.get(1));
		insertion.appendChild(bound);
	    }
	return insertion;
    }

    /**
     * Creates the subTree <deletion>. 
     * @return Element
     */
    public Element createDeletion()
    {
	Element deletion = document.createElement("Deletion");
	Vector deletionv = (Vector) resultDistribution.get(1);
	
	for(Enumeration e=deletionv.elements();e.hasMoreElements();)
	    {
		Vector story = (Vector) e.nextElement();
		Element bound = document.createElement("Boudary");
		TimeBase ts = (TimeBase) story.get(0);
		TimeBase tsn = ts.add(truth.getOffset());
		bound.setAttribute("TimeA",tsn.toHHMMssString());
		//bound.setAttribute("TimeAdebug",ts.toString());
		//bound.setAttribute("Type",(String)story.get(1));
		deletion.appendChild(bound);
	    }
	return deletion;
    }
    
    /**
     * it Creates the subTree <ClippedRef>.It displays all the boundaries clipped from the reference 
     * @return Element
     */
    public Element createClippedRef()
    {
	Element clippedRef = document.createElement("ClippedRef");
	Vector clippedRefv = (Vector) resultDistribution.get(3);
		
	for(Enumeration e=clippedRefv.elements();e.hasMoreElements();)
	    {
		Vector story = (Vector) e.nextElement();
		Element bound = document.createElement("Boundary");
		bound.setAttribute("TimeA",( ((TimeBase)story.get(0)).add(truth.getOffset()) ).toHHMMssString() );
		//bound.setAttribute("Type",(String)story.get(2));
		//bound.setAttribute("TimeADebugg",((TimeBase)story.get(0)).toString());
		clippedRef.appendChild(bound);
	    }
	return clippedRef;
    }

    /**
     * it Creates the subTree <ClippedSub>.It displays all the boundaries clipped from the submission 
     * @return Element
     */
    public Element createClippedSub()
    {
	Element clippedSub = document.createElement("ClippedSub");
	Vector clippedSubv = (Vector) resultDistribution.get(4);
	
	for(Enumeration e=clippedSubv.elements();e.hasMoreElements();)
	    {
		Vector story = (Vector) e.nextElement();
		Element bound = document.createElement("Boundary");
		bound.setAttribute("TimeD",((TimeBase)story.get(0)).toString());
		//bound.setAttribute("Type",(String)story.get(2));
		clippedSub.appendChild(bound);
	    }
	return clippedSub;
    }

   /**
     * It creates the subTree <referenceBeforeClipping> to display the reference before the clipping. used for debugging
     * @return Element
     */
    public Element createReference()
    {
	Element ref = document.createElement("ReferenceBeforeClipping");
	
	for(Enumeration e=truth.segmentationBeforeClipping.elements();e.hasMoreElements();)
	    {
	

		Vector story = (Vector) e.nextElement();
		Element bound = document.createElement("Story");

		TimeBase tb = (TimeBase) story.get(0);
		TimeBase talign = new TimeBase(tb.getint()-videosub.getAlign().getint());
		bound.setAttribute("bTimeA",tb.toString());
		bound.setAttribute("bTimeD",talign.toString());
		bound.setAttribute("Type",(String)story.get(2));
		ref.appendChild(bound);
	    }
	return ref;
    }

    /**
     * It creates the subTree <referenceAfterClipping> to display the reference after the clipping. used for debugging
     * @return Element
     */
    public Element createReferenceAfterClipping()
    {
	Element ref = document.createElement("ReferenceAfterClipping");
	
	for(Enumeration e=truth.segmentation.elements();e.hasMoreElements();)
	    {
	

		Vector story = (Vector) e.nextElement();
		Element bound = document.createElement("Story");

		TimeBase tb = (TimeBase) story.get(0);
		TimeBase talign = new TimeBase(tb.getint()-videosub.getAlign().getint());
		bound.setAttribute("bTimeA",tb.toString());
		bound.setAttribute("bTimeD",talign.toString());
		bound.setAttribute("Type",(String)story.get(2));
		ref.appendChild(bound);
	    }
	return ref;
    }

    /**
     * It creates the subTree <SubmissionBeforeClipping> to display the Submission before the clipping. used for debugging
     * @return Element
     */
    public Element createSubmission()
    {
	Element ref = document.createElement("SubmissionBeforeClipping");
	
	for(Enumeration e=videosub.segmentationBeforeClipping.elements();e.hasMoreElements();)
	    {
	

		Vector story = (Vector) e.nextElement();
		Element bound = document.createElement("Story");

		TimeBase tb = (TimeBase) story.get(0);
		TimeBase talign = new TimeBase(tb.getint()+videosub.getAlign().getint());
		bound.setAttribute("bTimeD",tb.toString());
		bound.setAttribute("bTimeA",talign.toString());
		bound.setAttribute("Type",(String)story.get(2));
		ref.appendChild(bound);
	    }
	return ref;
    }
    
    /**
     * It creates the subTree <SubmissionAfterClipping> to display the submission after the clipping. used for debugging
     * @return Element
     */
    public Element createSubmissionAfterClipping()
    {
	Element ref = document.createElement("SubmissionAfterClipping");
	
	for(Enumeration e=videosub.segmentation.elements();e.hasMoreElements();)
	    {
	

		Vector story = (Vector) e.nextElement();
		Element bound = document.createElement("Story");

		TimeBase tb = (TimeBase) story.get(0);
		bound.setAttribute("bTimeD",tb.toString());
		bound.setAttribute("bTimeA",tb.add(videosub.getAlign()).toString());
		bound.setAttribute("Type",(String)story.get(2));
		ref.appendChild(bound);
	    }
	return ref;
    }
}
