/**
 * This Class melts the Submission and the associated Truth to a a segment with multiple subsegments
 * SubSegment has only one type for the truth and one from the submission, thus it is easier to get the lenth of
 * Common story.
 * The times used are analogic
 */
import java.util.Vector;
import java.util.*;

public class Fusion
{

    private VideoSubmission sub;
    private TruthData truth;
    
    //Include all the boundaries
    private Vector time = new Vector(); // This Vector containe all the times of the submission and the truth
    private Vector timeOrder = new Vector(); // This Vector contains the ordering time from the Vector
    
    /**
     *Constructor method.This method add the time from dubmission and from the truth in the Vector time
     * <pre> 
     * Truth        |----|-|----|--|-----------|
     * Submission  |---|-----|-|---|-----|-----|
     * time        ||--|-|-|-|-||--|-----|-----|
     *</pre>
     * @param sub VideoSubmission object 
     *@param truth TruthData object associated to the submission
     *
     */
    public Fusion(VideoSubmission sub, TruthData truth)
    {
	//System.out.println("Object Fusion created");
	this.sub=sub;
	this.truth=truth;
	for(int i=1;i<sub.getNumberOfBoundaries()+1;i++)
	    {
		this.time.add(new TimeBase(sub.getTimeOfBoundary(i)+sub.getAlign().getint()));
	    }

	for(int j=1;j<truth.getNumberOfBoundaries()+1;j++)
	    {
		this.time.add(new TimeBase(truth.getTimeOfBoundary(j)));
	    }
	melt(time);
    }
    
    /**
     * This method is used to display on the standard oupout the contents of a Vector
     * @param timeOrder Vector
     */
    public void displayVector(Vector timeOrder)
    {
	String tmpString="";

	for(Enumeration e = timeOrder.elements();e.hasMoreElements();)
	    {
		TimeBase tmp = (TimeBase) e.nextElement();
		tmpString=tmpString+" "+tmp.getint();
	    }
	System.out.println("time : "+tmpString);
    }

    /**
     * This method melts in order way the Times from Submission and truthData. The Vector timeOrder is updated.
     * @param time Vector containing all the times
    */    
    public void melt (Vector time)
    {
	//System.out.println("melt function is called");
	TimeBase timeFirst = (TimeBase) time.get(0);
	int tref = timeFirst.getint();

	TimeBase lowest=timeFirst;	
	
	if(time.size() !=1)
	{
	    for(int i=1; i<time.size();i++)
		{
		    TimeBase timeT = (TimeBase) time.get(i);
		    int tTime = timeT.getint();
		    if (tTime<tref)
			{
			    tref =tTime;
			    lowest=timeT;
			}
		}

	    if(timeOrder.size() >0 )
		{
		    TimeBase lastElem = (TimeBase) timeOrder.lastElement();
		    if(lastElem.getint()!=tref)
			timeOrder.add(new TimeBase(tref));
		}
	    else
		timeOrder.add(new TimeBase(tref));
	    
	    time.remove(lowest);
	    melt(time);
	}
	else
	    {
		TimeBase time1 = (TimeBase) time.get(0);
		
		TimeBase lastElem = (TimeBase) timeOrder.lastElement();
		    if(lastElem.getint()!=tref)
			timeOrder.add(time1);
	    }
    }

    /**
     * This method get the commun length of a specific type of story. For TRECVID2003 it is "news"
     * @return TimeBAse Object
     * @param type type of the story
     */
    public TimeBase getLengthOfCommunStory(String type)
    {
	//System.out.println("getLengthOfCommunStory called for :"+type);

	TimeBase t1,t2;
	TimeBase result = new TimeBase(0);
	String typeSub, typeTruth;
	
	for(int i=0;i<timeOrder.size()-1;i++)
	    {
		t1= (TimeBase) timeOrder.get(i);
		t2= (TimeBase) timeOrder.get(i+1);
		typeSub = sub.getType(t1,t2);
		typeTruth = truth.getType(t1,t2);
		if(typeTruth.equals(type) && typeSub.equals(typeTruth))
		    {
			result = result.add(t1.getDifferenceBetween(t2));
		    }
	    }
	return result;
    }

    /**
     * This method returns a table of three Vectors useful for the ouput of the classification. Thus it returns 
     * <pre>
     *     | match |
     *     |nomatch|
     *     |missing|
     *</pre>
     * @return Vector[]
     * 
     */

    public Vector[] getOutputDistribution()
    {
	//System.out.println("getLengthOfCommunStory called for :"+type);

	TimeBase t1,t2;
	Vector[] result = new Vector[3];
	String typeSub, typeTruth;
	Vector matchtmp = new Vector();
	Vector nomatchtmp = new Vector();
	Vector missingtmp = new Vector();
	
	for(int i=0;i<timeOrder.size()-1;i++)
	    {
		t1= (TimeBase) timeOrder.get(i);
		t2= (TimeBase) timeOrder.get(i+1);
		typeSub = sub.getType(t1,t2);
		typeTruth = truth.getType(t1,t2);
		//System.out.println("Between t1 "+t1.getint()+" nnd t2 " +t2.getint()+"Submission :"+typeSub+" Truth : "+typeTruth);
		if(typeSub.equals(typeTruth) && !typeTruth.equals("none") && !typeSub.equals("none"))
		    {
			Vector tmp = new Vector();
 			tmp.add(t1.toString());
			tmp.add(t2.toString());
			tmp.add(typeSub);
			tmp.add(sub.getBeginStoryForTime(new TimeBase(t1.getint()-sub.offsetAnalogMinusDigital.getint())));
			tmp.add(truth.getBeginStoryForTime(t1));
			matchtmp.add(tmp);
						
		    }
	       	if ((typeTruth.equals("none") || typeSub.equals("none")) && !typeSub.equals(typeTruth))
		    {
			Vector tmp = new Vector();
			tmp.add(t1.toString());
			tmp.add(t2.toString());
			tmp.add(typeTruth);
			tmp.add(sub.getBeginStoryForTime(new TimeBase(t1.getint()-sub.offsetAnalogMinusDigital.getint())));
			tmp.add(truth.getBeginStoryForTime(t1));
			missingtmp.add(tmp); 
		    }
		if (!typeSub.equals(typeTruth) && !typeTruth.equals("none") && !typeSub.equals("none"))
		    {
			Vector tmp = new Vector();
			tmp.add(t1.toString());
			tmp.add(t2.toString());
			tmp.add(typeSub);
			tmp.add(typeTruth);
			tmp.add(sub.getBeginStoryForTime(new TimeBase(t1.getint()-sub.offsetAnalogMinusDigital.getint())));
			tmp.add(truth.getBeginStoryForTime(t1));
			nomatchtmp.add(tmp);
		    }
	    }
		
	result[0]=matchtmp;
	result[1]=nomatchtmp;
	result[2]=missingtmp;
	
	return result;
    }
}
