/**
 * This is an abstract Class. This class was created because the Reference and the submission must have the same structure to be compared.
 * This common structure is defined in this class
 */

import java.util.Vector;
import java.util.*;
import java.sql.Time;
import java.io.*;

public abstract class StoriesSegment
{
    final String datafile="../Data/"; //this variable is also defined in StorySegmentationEvaluation Class
    public Vector segmentation; // main attribute of this mother class, every Stories of a submission or a Truth is in this Vector
    public Vector segmentationBeforeClipping; // this attribute is more usefull for debugging
    private Vector oneStory; 
    final int numberOfRuns=10;
    public Vector clippingPoints;// vector of all the clipping Points
    public TimeBase lengthVideo;
    public boolean treated=true; // boolean attribute set to false when a video mustn't be treated
    public TimeBase offsetAnalogMinusDigital; //Alignment between Analogic and Digital Time. ie offsetAnalogMinusDigital = Analogic - Digital

    
    //The following Vectors are usefull for the output of the story segmentation task
    public Vector insertion = new Vector() ;
    public Vector deletion = new Vector() ;
    public Vector match = new Vector() ;
    public Vector clippedRef = new Vector();
    public Vector clippedSub = new Vector();
    
    //The following Vectors are usefull for the output of the Classification task
    public Vector matchType = new Vector() ;
    public Vector nomatchType = new Vector() ;
    public Vector missingType = new Vector() ;


    /**
     * Constructor method. It instantiates the segmentation Vector.
     * The structure of the segmentation Vector is as following :
     * <pre>
     *    |      |      |      |                        |      |
     *    |Vector|Vector|Vector| .......................|Vector|
     *    |      |      |      |                        |      |
     *       |
     *       |
     *
     *    | bTime  | : Begin Time of the Story
     *    | eTime  | : End Time of the Story, which is the begin time of the next one 
     *    | Type   | : Type of the story, for TRECVID2003 "misc" or "news"
     *</pre>
     */
    public StoriesSegment() 
    {
	segmentation = new Vector();
    }
    
    /**
     * returns the video file name of the current object either Truth or Submission
     * @return String 
     */
    public abstract String getVideoFileName();
    
    /**
     * this method add the end Time of the last Story ofthe segmentation Vector. 
     *Indeed the eTime of a story is always the beginning of the next story except for the last one.
     */
    public abstract void addEndTime();
    
    /**
     * Get the offset you have to apply on the a StoriesSegment Object Time to get the AnalogTime
     */
    public abstract TimeBase getAlign();
    
     /**
     * Return the length of the Mpeg1 video associated
     * @return TimeBase object, length of the video
     */
    public TimeBase getMPEG1Length()
    {
	LineNumberReader lnr;
	int nLine=0;
	Vector clip=new Vector();
	TimeBase length = new TimeBase(0);
	
	try{
	    
	    FileInputStream fis = new FileInputStream(new File(datafile+"lengthTableTestCollection.txt"));
	    InputStreamReader isr = new InputStreamReader(fis);
	    lnr = new LineNumberReader(isr);
	    	    
	    while(true)
		{
		    String line = lnr.readLine();
		    
		    if (line==null)
			break;
		    
		    String[] linesplit = line.split("\t");
		    String file = linesplit[0];
		    if(file.equals(getVideoFileName()))
		       length = new TimeBase(linesplit[1],"length");
				       
		}
	}catch (Exception e) {
	    System.out.println("error");
	    e.getMessage();
	    e.printStackTrace();
	}
	return length;
    }

    /**
     * This method applies the clipping points on a StoriesSegment object.
     *@return Vector it returns the clipped Stories.
     */
    public abstract Vector applyClippingPoints();

    /**
     * This method display on the standard ouput the content of the segmentation of a StoriesSegment Object.
     * Thus it will be be display like following :
     *   ...
     *   start : 28.03 end :54.02 Type : news
     *   start : 128.03 end :154.02 Type : misc
     *   ...
     */
    public void displaySegment()
    {
	for( Enumeration e = this.segmentation.elements(); e.hasMoreElements();)
	    {
		Vector tmp = (Vector) e.nextElement();
		TimeBase start = (TimeBase) tmp.get(0);
		TimeBase end = (TimeBase) tmp.get(1);
		String type = (String) tmp.get(2);
		//int length = end.getint() - start.getint();

		System.out.println("start : "+start+" end : " +end+ " Type : " + type);
	    }
    }    
   
    
    /**
     * This method return the type of a story between two boundaries. t1 t2 must be story boundaries from the submission or the Reference or also one from the submission and the other one from reference
     *... Warning : it works ONLY if t1 and t2 are boundaries and t1<t2
     *@return String type is "misc" or "news" for TRECVID2003
     *@param t1 first TimeBase object of a story boundary
     *@param t2 second timeBase object of a story boundary
     */
    public String getType(TimeBase t1, TimeBase t2)
    {
	String result="";
	boolean found=false;
	
	if(t2.getint()<t1.getint())
	    System.out.println("Error, the order Of the fusion vector is not good");
	else
	    {
	
		for(int i=0;i<getNumberOfStories();i++)
		    {
			if( t1.getint() >= getBeginTimeStory(i)+getAlign().getint() && t2.getint() <= getEndTimeStory(i)+getAlign().getint())
			    {
				result = getType(i);
				found=true;
				break;
			    }
		    }
	    }
	if(found)
	    return result;
	else
	    return "none";
    }

    /**
     * It returns the length of the current video (Mpeg1)
     *@return TimeBase
     *
     */
    public TimeBase getVideoLength()
    {
	
	TimeBase tmp = new TimeBase();
	tmp =  this.lengthVideo;
	return tmp;
    }
    
    /** 
     * It returns the clipping points ie the clipping start and the clipping end for the current Stories segment object.
     * If the clipping points are not found for this object, its attibute treated is set to false and this method return an empty Vector
     * @return Vector having the following structure
     *  <pre>          |fileName |
     *                 |clipstart|
     *                 |clipend  |
     *</pre>
     */
//     public Vector searchClippingPoints()
//     {
// 	Vector line= new Vector();
// 	boolean found=false;
	
// 	//System.out.println("size of the clipping point "+clippingPoints.size() );
// 	for(int i=0;i<clippingPoints.size();i++)
// 	    {
// 		Vector element = (Vector) clippingPoints.get(i);
// 		String namefile = (String) element.get(0);
// 		//System.out.println("this the name of the file at "+i+" : "+namefile);
// 		if(namefile.equals(getVideoFileName()))
// 		    {
// 			line = element;
// 			found =true;
// 			break;
// 		    }
// 	    }
// 	if(!found)
// 	    {
// 		System.out.println("Clipping Points not found for the video : "+getVideoFileName()+".This video won't be treated");
// 		//if there are no clippingpoints for this video, it mustm't be treated
// 		this.treated = false;
// 	    }
// 	return line;
//     }
    
    /**
     * Add a story to the SoriesSegment. The description of the story is in argument
     * @param beginTime the begin Time of the story to add
     * @param endTime the end time of the story to add
     * @param type type of the story to add
     */
    public void addOneStory(TimeBase beginTime, TimeBase endTime, String type)
    {
	oneStory = new Vector();
	oneStory.add(beginTime);
	oneStory.add(endTime);
	oneStory.add(type);
	segmentation.add(oneStory);
    }
    
    /**
     * This method invert the stories in the storiesSegment object. Therefore the last story becomes the first.
     * @return Vector . The inverted segmentation. 
     * @param segmentationInv the segmenttaion to invert.
     */
    public Vector invertStoriesSegment(Vector segmentationInv)
    {
	Vector tmpVector = new Vector();

	for(int i=1;i<segmentationInv.size()+1;i++)
	    {
		tmpVector.add(segmentationInv.get(segmentationInv.size()-i));
	    }
	return tmpVector;
    }
    
    
    /**
     * Returns the time of the 1st, 2nd ...n th boundary
     * @return int number of centiseconds
     * @param noBoundary int : numero of the boudary in the segmentation
     */
    public int getTimeOfBoundary(int noBoundary)
    {
	if(noBoundary==getNumberOfBoundaries())
	    {
		//System.out.println("number of stories : " + getNumberOfStories());
		//System.out.println("size of segmentation : "+ this.segmentation.size());
		//System.out.println("size of segmentation before clipping: "+ this.segmentationBeforeClipping.size());
		//this.segmentationBeforeClipping
		return getEndTimeStory(getNumberOfStories()-1);
	    }
	else
	    return getBeginTimeStory(noBoundary-1);
    }
    
    /**
     * returns the Begin Time of the story number nbSTory in the segmentation
     * @return int
     * @param nbStory int :no of story in the segmentation
     */
    public int getBeginTimeStory(int nbStory)
    {
	Vector ob = (Vector) this.segmentation.get(nbStory);
	TimeBase tmpint= (TimeBase) ob.get(0);
	return tmpint.getint();
    }
    
    /**
     * returns the End Time of the story number nbSTory in the segmentation
     * @return int
     * @param nbStory int :no of story in the segmentation
     */
    public int getEndTimeStory(int nbStory)
    {
	//System.out.println("get End time of " + nbStory);
	Vector ob = (Vector) this.segmentation.get(nbStory);
	TimeBase tmpint= (TimeBase) ob.get(1);
	return tmpint.getint();
    }
    
    /**
     * returns the type of the story number nbSTory in the segmentation
     * @return String : "news" or "misc" for TRECVID2003
     * @param nbStory int :no of story in the segmentation
     */
    public String getType(int nbStory)
    {
	Vector ob = (Vector) this.segmentation.get(nbStory);
	String tmp = (String) ob.get(2);
	return tmp;

    }
    /**
     * Gives a time t and this method will return the BeginTime of the story that time t belongs to
     * @return TimeBase begin Time of a story
     * @param t TimeBase Whose story is this time t? 
     */
    public TimeBase getBeginStoryForTime(TimeBase t)
    {
	TimeBase boundaryBegin= new TimeBase(0);
	for(int i=0;i<getNumberOfStories();i++)
	    {
		if( t.getint() >= getBeginTimeStory(i) && t.getint() < getEndTimeStory(i) )
		    {
			boundaryBegin = new TimeBase(getBeginTimeStory(i));	
		    }
	    }
	if (t.getint() >= getEndTimeStory(getNumberOfStories()-1))
	    {
		boundaryBegin = new TimeBase(getEndTimeStory(getNumberOfStories()-1));
	    }
	return boundaryBegin;
    }

    /**
     * Returns the number of Stories of the current StoriesSegment
     * @return int
     */
    public int getNumberOfStories()
    {
	return segmentation.size();
    }
    /**
     * Returns the number of boundaries of the current StoriesSegment
     * @return int
     */
    public int getNumberOfBoundaries()
    {
	return segmentation.size()+1;
    }
    
    /**
     * Returns the number of boundaries evaluated of the current StoriesSegment. Ie those inside the clipping points expect those ones
     * @return int
     */
     public abstract int getNumberOfEvaluatedBoundaries();
   //  {
// 	int result=0;

// 	Vector oneStory = new Vector();
// 	Vector line= new Vector();
// 	line = searchClippingPoints();
// 	TimeBase start = (TimeBase) line.get(1);
// 	TimeBase end = (TimeBase) line.get(2);
	
// 	for (int i=0;i<getNumberOfStories();i++)
// 	    {
// 		oneStory = (Vector) segmentation.get(i);
// 		TimeBase t0 = (TimeBase) oneStory.get(0);
// 		TimeBase t1 = (TimeBase) oneStory.get(1);
// 		if( !( (offsetAnalogMinusDigital.getint()<0 &&  t0.getint() == -offsetAnalogMinusDigital.getint()) || (offsetAnalogMinusDigital.getint()>0 &&  t0.getint() == offsetAnalogMinusDigital.getint())   ) )
// 		    result++;
// 		if(i==getNumberOfStories()-1)
// 		    {
// 			if( !( (180000 - offsetAnalogMinusDigital.getint() < this.lengthVideo.getint() && t1.getint() == 180000 - offsetAnalogMinusDigital.getint()) || ( this.lengthVideo.getint())+offsetAnalogMinusDigital.getint() < 180000 &&  t1.getint() == this.lengthVideo.getint()+ offsetAnalogMinusDigital.getint())  || t1.getint()==this.lengthVideo,getint())
// 			    result++;
// 		    }
// 	    }	
// 	return result;
//     }


    /**
     * Returns the length of story named typeStory of the current StoriesSegment
     * @return int total length od soty in the current StoriesSegment
     * @param typeStory type of the story. For TRECVID2003 it is "news"
     */
    public int getTotalTimeOneStory(String typeStory)
    {
	int duration=0;
	int i;
	String tempTypeStory;
	
	Vector oneStory = new Vector();
	
	
	for (i=0;i<getNumberOfStories();i++)
	    {
		oneStory = (Vector) segmentation.get(i);
		tempTypeStory = (String) oneStory.get(2);
		TimeBase t1 = (TimeBase) oneStory.get(1);
		TimeBase t0 = (TimeBase) oneStory.get(0);
		
		if (typeStory.equals(tempTypeStory))
		    { 
			//System.out.println(typeStory+" found");
			duration = duration + (t1.getint() - t0.getint());
			//System.out.println("time add sub : "+ duration.getint());
		    }
		else if(!tempTypeStory.equals("misc") && !tempTypeStory.equals("news"))
		{
		    System.out.println("Error in the submission structure, type must be misc or news");
		}
	    }
	return duration;
    }
    
    /**
     * Returns a Vector with all the Vectors useful ofr the Story boundary task
     * @return Vector The structure of this Vector is as following :
     * <pre>
     *         |insertion | : Vector containing the information for insertion story boundaries
     *         | deletion | : Vector containing the information for deletion story boundaries
     *         |  match   | : Vector containing the information for match story boundaries
     *         |ClippedRef| : Vector containing the information for Clipped story boundaries from the reference
     *         |ClippedSub| : Vector containing the information for Clipped story boundaries from the submission
     * </pre>
     */
    public Vector returnResultBoundaryDistribution()
    {
	Vector result = new Vector();
	
	result.add(insertion);
	result.add(deletion);
	result.add(match);
	result.add(clippedRef);
	result.add(clippedSub);

	return result;
    }

    /**
     * Returns a Vector with all the Vectors useful ofr the Classification task
     * @return Vector The structure of this Vector is as following :
     *<pre> 
     *         |matchtype  | : Vector containing the information for match stories
     *         |nomatch    | : Vector containing the information for storie that doesn't match
     *         |missingType| : Vector containing the information for stories which are in the Reference but not in the submission
     *         |ClippedRef | : Vector containing the information for Clipped stories from the reference
     *         |ClippedSub | : Vector containing the information for Clipped stories from the submission
     * </pre> 
     */
    public Vector returnResultStoryDistribution()
    {
	Vector result = new Vector();
		
	result.add(matchType);
	result.add(nomatchType);
	result.add(missingType);
	result.add(clippedRef);
	result.add(clippedSub);

	return result;
    }

 
 


}
