/**
 *Main Class of the Evalutation program of the Story segmentation task which is a new task of TRECVID2003
 *This class has only one argument and it is the directory of the group you want to evaluate...see the comment of the constructor
 *@author Cedric COULON {@link cedric.coulon@nist.gov}
 *@version 1.0
 */

import java.util.Vector;
import java.io.File;
import org.w3c.dom.*;
import com.sun.xml.tree.*;


import java.util.*;
import java.io.*;

public class StorySegmentationEvaluation
{
    Document document = new XmlDocument();
    Element resultStory = document.createElement("ResultStorySegmentationEvaluation");
    
    private double fuzzyfactor;
    public Vector listTruth;
    
    final File directoryTruth = new File("../Reference/");
    final String pathSubmission ="../Submissions/";
    final String pathResult="../Result/";
    final String datafile="../Data/"; //this variable is also defined in VideoSubmission Class

    private String nameGroup;
    
    final int numberOfRun=10;
    Submission sub;
    float[] storyBoundaryRecallRun = new float[numberOfRun];
    float[] storyBoundaryPrecisionRun = new float[numberOfRun];
    float[] newsSegmentRecallRun = new float[numberOfRun];
    float[] newsSegmentPrecisionRun = new float[numberOfRun];
    //Vector clippingPoints; //Vector containing all the clipping points of the videos
    Vector offsets; // Vector containing all the offsets of the videos
    Vector listTruthCopy=new Vector(); //Vector containing all thre reference objects (TruthData objects)
    int totalLengthOfNewsInTruth=0;
    int totalLengthOfMiscInTruth=0;
    int totalNumberOfRefBoundaries=0;
    
    /**
     *Constructor of the Class. it loops on the files within the directoris you give in paramter
     *@param directorySubmissionStr The name of the directory you want to be evaluated. This directory must be in the path indicated by the attribute pathsubmission.
     *                              The name of the directory should be the name of the group. Afterwards this name will be concatenated to the result file. 
     *                              Indeed if you would like to evalute the result of the nist it will create a file named Resultnist.xml into the directory given in the attribute pathResult
     */
    public StorySegmentationEvaluation(double fuzzyfactor, String directorySubmissionStr)
    {
	this.fuzzyfactor=fuzzyfactor;
	listTruth = loadTruthData();
	nameGroup=directorySubmissionStr;
	document.appendChild(resultStory);
	//clippingPoints = loadClippingPoints();
	offsets = loadOffsets();
	File directorySubmission = new File(pathSubmission+directorySubmissionStr);
	if(!directorySubmission.isDirectory())
	    System.out.println("The path added in argument is not a directory");
	else
	    {
		totalLengthOfNewsInTruth = getGlobalTotalTimeofOneStoryAllTruth("news");
		totalLengthOfMiscInTruth = getGlobalTotalTimeofOneStoryAllTruth("misc");
		totalNumberOfRefBoundaries= getGlobalNumberOfEvaluatedBoundariesinAllTruth();

		File[] list =directorySubmission.listFiles(new GenericFileFilter("*.xml"));
		resultStory.setAttribute("NumberBoundariesEvaluatedInAllTruth",Integer.toString(totalNumberOfRefBoundaries  ));
		resultStory.setAttribute("TotalLengthOfNewsInAllTruth",(new TimeBase(totalLengthOfNewsInTruth)).toString());
		resultStory.setAttribute("TotalLengthOfMiscInAllTruth",(new TimeBase(totalLengthOfMiscInTruth)).toString());

		//loop on every xml file in the directory you put in directorySubmissionStr
		for(int i=0;i<list.length;i++)
		    {
			System.out.println("\n"+list[i].getName()+" in process...\n");
			sub = new Submission(list[i]);
			
			Vector runs = getRunsEvaluated();
			for( Enumeration e = runs.elements(); e.hasMoreElements();)
			    {
				Element runEl = (Element) e.nextElement();
				resultStory.appendChild(runEl);
				
			    }
		    }
		
		write();
	    }
    }

    /**
     * This function is called in the constructor of this class. It returns a Vector of Elements
     *@return A vector of Element
     */
    public Vector getRunsEvaluated()
    {
	Vector vectorRunElement=new Vector(); // Contains Element objects for the ouput of each run
	Node run, video;
	VideoSubmission videosub;
	TruthData truth;
	int nbBoundDet, nbBoundRef, nbBoundSub,nbTotalTimeIdentifiedNews,nbTotalNewsSubmission, nbTotalNewsReference,nbBoundDetSub ;
	int nbVideoTreated;
	
	//loop on each submitted run
	for(int i=0; i<getNumberOfRunSubmitted(sub);i++)
	    {
		
		//variable usefull to create trees for the story boundary task and the classification one
		DistributionTree dis;
		StoryTree disStory;
		
		nbBoundDet = 0;
		nbBoundRef = 0;
		nbBoundSub = 0;
		nbTotalTimeIdentifiedNews=0;
		nbTotalNewsSubmission =0;
		nbTotalNewsReference =0;
		nbBoundDetSub =0;
		nbVideoTreated = 0;
				
		//Creation of the tree for one run
		Element runEl =document.createElement("Run");
		Element boundaries = document.createElement("Boundaries");
		Element stories = document.createElement("Stories");
		
		run = (Node) sub.getRun().get(i);
		int nbVideo = getNumberOfVideoSubmitted(run);
		if(nbVideo != listTruth.size())
		    System.out.println("The number of the videos contained in that run is not equal to the number of sources, however this run is gonna be evaluated");
		
		Element videosMissing = getVideosMissing(run);
		runEl.appendChild(videosMissing);

		//loop on each video
		for(int j=0; j<nbVideo;j++)
		    {
			//To optimize, if you dont reload all the truth, the evaluation will be done on clipped truth 
			listTruth = loadTruthData();
			

			Vector resultDistibution = new Vector();
			// gets the the video of the current run Node
			video=sub.getVideo(j,run);
			//it creates VideoSubmission that will load these video results
			videosub = new VideoSubmission(sub.getTyped(run),video,offsets);
			
			//if this video can be treated. ie we have an offset, clipping points
			if (videosub.treated)
			    {
				nbVideoTreated++;
				
				truth = getTruth(videosub);
					
				if (truth==null)
				    System.out.println("The src_sgm file wasn't found for this video");
				
				
				videosub.setTruthData(truth);
				if(truth.treated)
				    {
					dis = new DistributionTree(videosub,truth, document);
					Element videoDistributionTree = dis.getTree();
					boundaries.appendChild(videoDistributionTree);
					
					runEl.appendChild(boundaries);
					
					nbBoundSub = nbBoundSub + videosub.getNumberOfEvaluatedBoundaries();						    
					nbBoundDet = nbBoundDet + videosub.getNumbersOfReferenceBoundariesDetectedOneFile(fuzzyfactor);
					nbBoundRef = nbBoundRef + truth.getNumberOfEvaluatedBoundaries();
					nbBoundDetSub = nbBoundDetSub + videosub.getNbDetectedBoundaries(fuzzyfactor);
					
					if (sub.getTyped(run))
					    {
						//System.out.println("Time of Identified News in sub : "+videosub.getTotalTimeIdentifiedOf("news"));
						nbTotalTimeIdentifiedNews = nbTotalTimeIdentifiedNews + videosub.getTotalTimeIdentifiedOf("news");
						
						//System.out.println("Time of News in sub            : "+videosub.getTotalTimeOneStory("news"));
						nbTotalNewsSubmission = nbTotalNewsSubmission + videosub.getTotalTimeOneStory("news");
						
						//System.out.println("Time of News in truth          : "+truth.getTotalTimeOneStory("news"));
						nbTotalNewsReference = nbTotalNewsReference + truth.getTotalTimeOneStory("news");
										    
						disStory = new StoryTree(videosub,truth, document);
						Element storyDistributionTree = disStory.getTree();
						stories.appendChild(storyDistributionTree);
					    }
					
					runEl.appendChild(stories);
				    }
			    }
		    }
		//System.out.println("nbBoundDetSub   " + nbBoundDetSub);
		//System.out.println("nbBoundDet Refe " + nbBoundDet);
		//System.out.println("nbBoundSub      " + nbBoundSub);
		//System.out.println("nbBoundRef      " + nbBoundRef);
		//System.out.println("nbTotalNewsSub " + nbTotalNewsSubmission);
		//System.out.println("nbTotalTimeIdentifiedNews " + nbTotalTimeIdentifiedNews);
		//System.out.println("nbBoundRef " + nbBoundRef);
		//System.out.println("nbBoundDet/nbBoundRef" + (float)((float)nbBoundDet/(float)nbBoundRef));
		
		//The goal of the first comment line is to figured out the recall from the results provided and only provided by the group. The second line gives the recall, but this time it counts everything in the truth 
		//Therefore tahnks to the second lines the groups who didn't provide a Result for each video of the test collection will be punished by a lower Recall

		//storyBoundaryRecallRun[i] = (float)nbBoundDet / (float)nbBoundRef;
		storyBoundaryRecallRun[i] = (float)nbBoundDet / (float) totalNumberOfRefBoundaries;
		storyBoundaryPrecisionRun[i] = (float) nbBoundDetSub / (float)nbBoundSub;
		
		//System.out.println("here is the recall for the run "+i+"    : "+ storyBoundaryRecallRun[i]+"\n");
		//System.out.println("here is the precision for the run "+i+" : "+ storyBoundaryPrecisionRun[i]+"\n");

		if(sub.getTyped(run))
		    {
			
			// it is the same explanation as above, but this time for the Classification task results
			//newsSegmentRecallRun[i] = (float)nbTotalTimeIdentifiedNews / (float) nbTotalNewsReference ;
			
			newsSegmentRecallRun[i] = (float)nbTotalTimeIdentifiedNews / (float) totalLengthOfNewsInTruth;
			newsSegmentPrecisionRun[i] = (float)nbTotalTimeIdentifiedNews / (float) nbTotalNewsSubmission ;
						
			//System.out.println("here is the News recall for the run "+i+"    : "+ newsSegmentRecallRun[i]+"\n");
			//System.out.println("here is the News precision for the run "+i+" : "+ newsSegmentPrecisionRun[i]+"\n");
		    }

		String typ;
		if(sub.getTyped(run))
		    typ="yes";
		else
		    typ="no";

		String cond = Integer.toString(sub.getCondition(run));
		String prio = Integer.toString(sub.getPriority(run));

		runEl.setAttribute("SysId",cond+"_"+sub.getSysId(run)+"_"+prio);
		runEl.setAttribute("Condition",cond);
		runEl.setAttribute("Priority",prio);
		//runEl.setAttribute("Priority",Integer.toString(i+1));
		runEl.setAttribute("Typed",typ);
		runEl.setAttribute("NumberOfVideosMissing",Integer.toString(listTruth.size()-nbVideoTreated));
		runEl.setAttribute("NumberBoundariesEvaluatedInSubmission",Integer.toString(nbBoundSub));
		/*Uncomment the follwing line to add the number of boudaries in truth only for the result provided. 
		  ie, if a group provides a result for each video nnBoundRef = getGlobalNumberOfEvaluatedBoundariesinAllTruth()*/
		//runEl.setAttribute("NumberBoundariesEvaluatedInTruth",Integer.toString(nbBoundRef));
		runEl.setAttribute("NumberBoundariesFoundInSub",Integer.toString(nbBoundDetSub));
		runEl.setAttribute("NumberBoundariesFoundInTruth",Integer.toString(nbBoundDet));
		runEl.setAttribute("Recall",Float.toString(storyBoundaryRecallRun[i]));
		runEl.setAttribute("Precision",Float.toString(storyBoundaryPrecisionRun[i]));
		if(sub.getTyped(run))
		    {
			
			runEl.setAttribute("TotalLengthOfNewsInSubmission",(new TimeBase(nbTotalNewsSubmission)).toString());
			//runEl.setAttribute("TotalLengthOfNewsInTruth",(new TimeBase(nbTotalNewsReference)).toString());
			runEl.setAttribute("LengthOfNewsIdentifiedInSubmission",(new TimeBase(nbTotalTimeIdentifiedNews)).toString());
			runEl.setAttribute("NewsRecall",Float.toString(newsSegmentRecallRun[i]));
			runEl.setAttribute("NewsPrecision",Float.toString(newsSegmentPrecisionRun[i]));	
		    }
		
		vectorRunElement.add(runEl);
		
	    }
	
	
	return vectorRunElement;
	

    }
    
    /**
     * This function returns the offset you must apply on the submission file corresponding to the truth file.
     * To apply the clipping points it's important to kmow the clipping points anf the o
     *@return the offset you apply on the submission 
     *@param truth 
     */

    public TimeBase getAlignementOfPossibleSubmission(TruthData truth)
    {
	TimeBase off= new TimeBase(0);
	boolean found = false;	
	
	for(int i=0;i<offsets.size();i++)
	    {
		Vector element = (Vector) offsets.get(i);
		String namefile = (String) element.get(0);
		if(namefile.equals(truth.getVideoFileName()))
		    {
			off = (TimeBase) element.get(1);
			found = true;
			break;
		    }
	    }
	if (found==false)
	    {
		truth.treated=false;
	    }
	
	return off;	
    }

    /**
     * This Method figures out the total number of boundaries evaluted in every files of the reference
     * @return it returns an int, number of boundaries evaluated in the whole truth
     *
     */
    public int getGlobalNumberOfEvaluatedBoundariesinAllTruth()
    {
	int result=0;
	
	Vector listTruthTmp = (Vector) copylistTruth(listTruth).clone();
	for(int i=0;i<listTruthTmp.size();i++)
	    {
		TruthData truthnew = (TruthData)((TruthData) listTruthTmp.get(i)).clone();
		//truthnew.clippingPoints = clippingPoints;
		truthnew.offsetAnalogMinusDigital=getAlignementOfPossibleSubmission(truthnew);
		if (truthnew.treated)
		    {
			truthnew.applyClippingPoints();
			result = result + truthnew.getNumberOfEvaluatedBoundaries();
		    }
	    }
	return result;
    }

    public Vector copylistTruth(Vector v)
    {
	Vector result = new Vector();
	for( Enumeration e = v.elements(); e.hasMoreElements();)
	    {
		result.add(((TruthData) e.nextElement()).clone());
	    }
	return result;
    }

    /**
     * This Method figures out the length of a story type in every files of the reference
     * @return it returns an int, length of a type
     * @param typeStory the type of the story we are looking for, for TRECVID2003 it was "news"
     */
    public int getGlobalTotalTimeofOneStoryAllTruth(String typeStory)
    {
	int result=0;
	Vector listTruthTmp =(Vector) copylistTruth(listTruth).clone();
	for(int i=0;i<listTruthTmp.size();i++)
	    {
		TruthData truthnew1 = (TruthData)((TruthData) listTruthTmp.get(i)).clone();
		//truthnew1.clippingPoints = clippingPoints;
		truthnew1.offsetAnalogMinusDigital=getAlignementOfPossibleSubmission(truthnew1);
		truthnew1.applyClippingPoints();
		result = result + truthnew1.getTotalTimeOneStory(typeStory);
	    }
	return result;	
    }

    /**
     *This method write an xml file into the directory Result you put in argument of the execution command. 
     *if the directory containing the submission is name xxx, the result file will be named Resulxxx.xml and
     * it will be placed in the Result directory
     */
    public void write()
    {
	try {
	    String file=pathResult+"Result"+nameGroup+".xml";
	    ((XmlDocument) document ).write(new FileWriter(file),"ISO-8859-1");
	} catch(Exception e) {
            e.printStackTrace();
        }
    }
    /**
     * This methos returns the name of the Videos Missing in a run of the submission
     *@return it returns an XML Element, it is the <videoMissing> tag
     */
    public Element getVideosMissing(Node run)
    {
	Element vmiss = document.createElement("VideosMissing");
	
	for(int i=0;i<listTruth.size();i++)
	    {
		boolean found=false;
		Node video;
		int nbVideo = getNumberOfVideoSubmitted(run);	
		
		String nameTruth = ( (TruthData) listTruth.get(i) ).getVideoFileName();
		for(int j=0; j<nbVideo;j++)
		    {
			video=sub.getVideo(j,run);
			VideoSubmission videosub = new VideoSubmission(sub.getTyped(run),video);
			if( nameTruth.equals(videosub.getVideoFileName()) )
			    {
				found = true;
				break;
			    }
		    }
		
		if(!found)
		    {
			Element videoName = document.createElement("VideoMissing");
			videoName.setAttribute("Name",nameTruth);
			vmiss.appendChild(videoName);
		    }
		
	    }
	return vmiss;
    }

    /**
     * Returns the number of the runs submitted in an Submission ie one xml file
     * @return it returns an int
     * @param submi its is a submission object wich is the Object containing all usefull information for one XML file
     */

    public int getNumberOfRunSubmitted(Submission submi)
    {
	Node sub = submi.getRoot();

	int result = 0;

	NodeList list = sub.getChildNodes();
	for(int i=0;i<list.getLength();i++)
	    {
		if(list.item(i).getNodeName().equals("storySegmentationRunResult"))
		    result++;
	    }
	if(result==0)
	    System.out.println("Impossible to find Videos in that run, please check the xml structure, you must find the element <storySegmentationRunResult>");
	return result;
    }

    /**
     * This method returns the number of videos in on specific run
     *@return it returns an int
     *@param run it is the the XML Node of one run
     */
    public int getNumberOfVideoSubmitted(Node run)
    {
	int result = 0;

	NodeList list = run.getChildNodes();
	for(int i=0;i<list.getLength();i++)
	    {
		if(list.item(i).getNodeName().equals("fileResult"))
		    result++;
	    }
	if(result==0)
	    System.out.println("Impossible to find Videos in that run, please check the xml structure, you must find the element <fileResult>");
	return result;
	
    }

    /**
     * This method get the extension of a file
     *@return it returns the extension as a string
     *@param f the you want to get the extension from
     */
    public String getExtension(File f) {
     if(f != null) {
         String filename = f.getName();
         int i = filename.lastIndexOf('.');
         if(i>0 && i<filename.length()-1) {
         return filename.substring(i+1).toLowerCase();
         };
     }
     return null;
     }
    
    /**
     * This method is used to get the TruthData object corresponding to the video. ie its finds the reference for a specific video
     *@return TruthData object, the reference object you have to compare the video to
     *@param vidsub a VideoSubmission object, it is actually a object contaning the result for One video of One specific run of One group
     */
    public TruthData getTruth(VideoSubmission vidsub)
    {
	System.out.println("search Truth for this video file");

	String nameVideo = vidsub.getVideoFileName();
	TruthData tmp = new TruthData();
	int found=0;

	for(int i=0;i<listTruth.size();i++)
	    {
		TruthData truth = (TruthData) listTruth.get(i);
		//System.out.println("file for this truth : "+truth.getFileName());
		if(truth.getVideoFileName().equals(nameVideo))
		{
		    System.out.println("Truth found for the video : "+truth.getVideoFileName());
		    tmp=truth;
		    found=1;
		    break;
		}  
				    
	    }
	if (found==0)
	    System.out.println("impossible to find the truth data for the video : "+ nameVideo );
	return tmp;
    }
    
    /**
     * It loads all the Reference file within the directory directoryTruth, an attribute of this class
     * @return TruthData Object is created for each file and is added to a vector which is returned
     */
  
    public Vector loadTruthData()
    {
	Vector listTruthtmp = new Vector();

	//System.out.println("Loading Truth dada from the src_sgm directory");
	File[] listTruthFile= new File[300];

	if (!directoryTruth.isDirectory())
	    System.out.println("the directory of the the src_sgm is not correct");
	else
	    {
		listTruthFile = directoryTruth.listFiles();
		for(int i=0; i<listTruthFile.length;i++)
		    {
			if(getExtension(listTruthFile[i]).equals("src_sgm"))
			{
			    listTruthtmp.add(new TruthData(listTruthFile[i]));
			    			    
			}
		    }
	    }

	return listTruthtmp;
    }

 
    /**
     * It load All the offset fron the alignment table named "alignment.txt.final".
     *@return it returns a Vector containing sub-Vector. Those sub-Vector contains the offset and the name of the file the offset has to be applied
     */

    public Vector loadOffsets()
    {
	System.out.println("Loading Offsets");
	LineNumberReader lnr;
	int nLine=0;
	Vector off=new Vector();

	try{
	    
	    FileInputStream fis = new FileInputStream(new File(datafile+"alignment.txt.final"));
	    InputStreamReader isr = new InputStreamReader(fis);
	    lnr = new LineNumberReader(isr);
	    
	    
	    while(true)
		{
		    String line = lnr.readLine();
		    		    
		    if (line==null)
			break;
		    if (!line.startsWith("-") && !line.startsWith("name") && !line.startsWith("File"))
			{
			    Vector tmp=new Vector();
			    String[] linesplit = line.split("\\s+");
			    
			    String file = linesplit[0];
			    //System.out.println(linesplit[0]+"\t"+linesplit[1]);
			    //time for a submission is like ss.xx
			    
			    TimeBase offset = new TimeBase(linesplit[1],"alignment");
			    tmp.add(file);
			    tmp.add(offset);
			    off.add(tmp);
			}
		}
	}catch (Exception e) {
	    System.out.println("error in loading offsets");
	    e.getMessage();
	    e.printStackTrace();
	    System.exit(0);
	}
	return off;
    }

    
    static public void main(String[] args)
    {
	if (args.length !=2)
	    System.out.println(args.length + " Usage : java StorySegmentationEvaluation [fuzzyfactor] [directory of the submission] ");
	else
	    new StorySegmentationEvaluation(Double.parseDouble(args[0]), args[1]);
    }

}
