/**
 * This class is used to build an Xml Node for the Classification task output.
 * Each object of this class contains the result for a video result of one specific Run
 * and only for the classification task
 */

import java.io.*;
import org.w3c.dom.*;
import java.util.*;
import com.sun.xml.tree.*;

public class StoryTree
{
    Document document = new XmlDocument();
    VideoSubmission videosub;
    TruthData truth;
    Vector resultDistribution = new Vector();
    Element video;// Xml Element, this element will be addes to the xml file result in StoriesSegmentEvalutionClass
    String fileName;
    TimeBase clippingStart, clippingEnd, length;
    int timeIdentifiedNews,timeNewsSubmission,timeNewsReference,totalTimeNewsReference;
    float recall,precision;
    Vector listTruth,offsets;
   

    /**
     * Constructor method.
     * @param vid VideoSubmission  
     * @param truth truthData associated to vid
     * @param doc Document, the documnet to wich these information are going to be added
     */
    StoryTree(VideoSubmission vid, TruthData truth, Document doc)
    {
	timeIdentifiedNews=timeNewsSubmission=timeNewsReference=totalTimeNewsReference=0;

	this.videosub = vid;
	this.truth = truth;
	this.document=doc;
	
	// Get all the results for this video
	resultDistribution = videosub.returnResultStoryDistribution();
	fileName = videosub.getVideoFileName();
	//clippingStart = (TimeBase) videosub.searchClippingPoints().get(1);
	//clippingEnd = (TimeBase) videosub.searchClippingPoints().get(2);
	length = videosub.getVideoLength();
	timeIdentifiedNews = videosub.getTotalTimeIdentifiedOf("news");
	timeNewsSubmission = videosub.getTotalTimeOneStory("news");
	timeNewsReference= truth.getTotalTimeOneStory("news");
	//totalTimeNewsReference= totalLengthOfNews;
	recall = (float)timeIdentifiedNews / (float) timeNewsReference ;
	precision = (float) timeIdentifiedNews / (float) timeNewsSubmission ;

	createTree();
    }
    
     /**
     * returns the complete Tree containing the results of the classification task for the VideoSubmission
     * @return Element, Xml Element
     */
    public Element getTree()
    {
	return video;
    }

    /**
     * This method creates the tree, set the attributes and the elements
     * @return void
     */ 
    public void createTree()
    {
	TimeBase tr = new TimeBase(timeNewsReference);
	TimeBase ts = new TimeBase(timeNewsSubmission);
	TimeBase ti = new TimeBase(timeIdentifiedNews);
	//TimeBase ttr = new TimeBase(totalTimeNewsReference);
	
	video   = document.createElement("video");
	video.setAttribute("src",fileName);
	video.setAttribute("length",length.toString());
	//video.setAttribute("clippingStart",clippingStart.toString());
	//video.setAttribute("clippingEnd",clippingEnd.toString());
	video.setAttribute("Offset",videosub.getAlign().toString());
	video.setAttribute("TotalTimeNewsInRef",tr.toString());
	//video.setAttribute("TotalTimeNewsInAllRef",ttr.toString());
	video.setAttribute("TotalTimeNewsSub",ts.toString());
	video.setAttribute("TotalTimeNewsIdentified",ti.toString());
	video.setAttribute("Recall",Float.toString(recall));
	video.setAttribute("Precision",Float.toString(precision));
	//video.setAttribute("alignement",truth.offsetAnalogMinusDigital.toString());
	
	Element match = createMatch();
	Element nomatch = createNoMatch();
	Element missing = createMissing();

	Element clippedRef = createClippedRef();
	Element clippedSub = createClippedSub();
	Element reference = createReference();
	Element referenceAfter = createReferenceAfterClipping();
	Element submission = createSubmission();
	Element submissionAfter = createSubmissionAfterClipping();

	//Uncomment theses two following lines to get the Submission and the Reference in the Result file
// 	video.appendChild(reference);
// 	video.appendChild(referenceAfter);
// 	video.appendChild(submission);
// 	video.appendChild(submissionAfter);

	video.appendChild(clippedSub);
	video.appendChild(clippedRef);
	video.appendChild(match);
	video.appendChild(nomatch);
	video.appendChild(missing);
    }
    
    /**
     * Create the subTree <match>. 
     * @return Element
     */
    public Element createMatch()
    {
	Element matchE = document.createElement("Match");
	Vector matchv = (Vector) resultDistribution.get(0);
	
	for(Enumeration e=matchv.elements();e.hasMoreElements();)
	    {
		Vector story = (Vector) e.nextElement();
		Element boundstory = document.createElement("Seg");
		boundstory.setAttribute("bTimeA",(String)story.get(0));
		boundstory.setAttribute("eTimeA",(String)story.get(1));
		boundstory.setAttribute("Type",(String)story.get(2));
		String storyBeginSub=((TimeBase)story.get(3)).toString();
		boundstory.setAttribute("StorybTimeDSub",storyBeginSub);
		//String storyBeginSubAlign=((TimeBase)story.get(5)).toString();
		//boundstory.setAttribute("StorybTimeSubAlign",storyBeginSubAlign);
		String storyBeginRef=(((TimeBase)story.get(4)).add(truth.getOffset())).toHHMMssString();
		boundstory.setAttribute("StorybTimeARef",storyBeginRef);

		matchE.appendChild(boundstory);
	    }
	return matchE;
    }

    /**
     * Create the subTree <noMatch>. 
     * @return Element
     */
    public Element createNoMatch()
    {
	Element nomatchE = document.createElement("noMatch");
	Vector nomatchv = (Vector) resultDistribution.get(1);
	
	for(Enumeration e=nomatchv.elements();e.hasMoreElements();)
	    {
		Vector story = (Vector) e.nextElement();
		Element boundstory = document.createElement("Seg");
		boundstory.setAttribute("bTimeA",(String)story.get(0));
		boundstory.setAttribute("eTimeA",(String)story.get(1));
		boundstory.setAttribute("TypeSub",(String)story.get(2));
		boundstory.setAttribute("TypeRef",(String)story.get(3));
		String storyBeginSub=((TimeBase)story.get(4)).toString();
		boundstory.setAttribute("StorybTimeDSub",storyBeginSub);
		//String storyBeginSubAlign=((TimeBase)story.get(5)).toString();
		//boundstory.setAttribute("StorybTimeSubAlign",storyBeginSubAlign);
		String storyBeginRef=(((TimeBase)story.get(5)).add(truth.getOffset())).toHHMMssString();
		boundstory.setAttribute("StorybTimeARef",storyBeginRef);

		nomatchE.appendChild(boundstory);
	    }
	return nomatchE;
    }

    /**
     * Create the subTree <mising>. The stories are in the truth but not in the submission 
     * @return Element
     */
    public Element createMissing()
    {
	Element missingE = document.createElement("MissingInSubmission");
	Vector missingv = (Vector) resultDistribution.get(2);
	
	for(Enumeration e=missingv.elements();e.hasMoreElements();)
	    {
		Vector story = (Vector) e.nextElement();
		Element boundstory = document.createElement("Se");
		boundstory.setAttribute("bTimeA",(String)story.get(0));
		boundstory.setAttribute("eTimeA",(String)story.get(1));
		boundstory.setAttribute("TypeRef",(String)story.get(2));
		String storyBeginSub=((TimeBase)story.get(3)).toString();
		boundstory.setAttribute("StorybTimeDSub",storyBeginSub);
		//String storyBeginSubAlign=((TimeBase)story.get(5)).toString();
		//boundstory.setAttribute("StorybTimeSubAlign",storyBeginSubAlign);
		String storyBeginRef=(((TimeBase)story.get(4)).add(truth.getOffset())).toHHMMssString();
		boundstory.setAttribute("StorybTimeARef",storyBeginRef);

		missingE.appendChild(boundstory);
	    }
	return missingE;
    }
 
    /**
     * it Creates the subTree <ClippedRef>.It displays all the Stories clipped from the reference 
     * @return Element
     */
    public Element createClippedRef()
    {
	Element clippedRef = document.createElement("ClippedRef");
	Vector clippedRefv = (Vector) resultDistribution.get(3);
	
	//System.out.println("size of the vector clipped ref "+ clippedRefv.size());
	
	for(Enumeration e=clippedRefv.elements();e.hasMoreElements();)
	    {
		Vector story = (Vector) e.nextElement();
		
		Element bound = document.createElement("Seg");
		bound.setAttribute("bTimeA",( ((TimeBase)story.get(0)).add(truth.getOffset()) ).toHHMMssString());
		bound.setAttribute("eTimeA",( ((TimeBase)story.get(1)).add(truth.getOffset()) ).toHHMMssString());
		//bound.setAttribute("bTimeADebug",((TimeBase)story.get(0)).toString());
		//bound.setAttribute("eTimeADebug",((TimeBase)story.get(1)).toString());
		bound.setAttribute("Type",(String)story.get(2));
		clippedRef.appendChild(bound);
		
	    }
	return clippedRef;
    }

    /**
     * it Creates the subTree <ClippedSub>.It displays all the stories clipped from the submission 
     * @return Element
     */
    public Element createClippedSub()
    {
	Element clippedSub = document.createElement("ClippedSub");
	Vector clippedSubv = (Vector) resultDistribution.get(4);
	
	for(Enumeration e=clippedSubv.elements();e.hasMoreElements();)
	    {
		Vector story = (Vector) e.nextElement();
		
		Element bound = document.createElement("Seg");
		bound.setAttribute("bTimeD",((TimeBase)story.get(0)).toString());
		bound.setAttribute("eTimeD",((TimeBase)story.get(1)).toString());
		bound.setAttribute("Type",(String)story.get(2));
		clippedSub.appendChild(bound);
	    }
	return clippedSub;
    }

   /**
     * It creates the subTree <referenceBeforeClipping> to display the reference before the clipping. used for debugging
     * @return Element
     */
    public Element createReference()
    {
	Element ref = document.createElement("ReferenceBeforeClipping");
	
	for(Enumeration e=truth.segmentationBeforeClipping.elements();e.hasMoreElements();)
	    {
	

		Vector story = (Vector) e.nextElement();
		Element bound = document.createElement("Story");

		TimeBase tb = (TimeBase) story.get(0);
		TimeBase te = (TimeBase) story.get(1);
		TimeBase talign = new TimeBase(tb.getint()-videosub.getAlign().getint());
		bound.setAttribute("bTimeA",tb.toString());
		//bound.setAttribute("bTimeD",talign.toString());
		//bound.setAttribute("eTimeA",te.toString());
		bound.setAttribute("Type",(String)story.get(2));
		ref.appendChild(bound);
	    }
	return ref;
	
    }
    
    /**
     * It creates the subTree <referenceAfterClipping> to display the reference after the clipping. used for debugging
     * @return Element
     */
     public Element createReferenceAfterClipping()
    {
	Element ref = document.createElement("ReferenceAfterClipping");
	
	for(Enumeration e=truth.segmentation.elements();e.hasMoreElements();)
	    {
	

		Vector story = (Vector) e.nextElement();
		Element bound = document.createElement("Story");

		TimeBase tb = (TimeBase) story.get(0);
		TimeBase te = (TimeBase) story.get(1);
		TimeBase talign = new TimeBase(tb.getint()-videosub.getAlign().getint());
		bound.setAttribute("bTimeA",tb.toString());
		//bound.setAttribute("bTimeD",talign.toString());
		//bound.setAttribute("eTimeA",te.toString());
		bound.setAttribute("Type",(String)story.get(2));
		ref.appendChild(bound);
	    }
	return ref;
	
    }

    /**
     * It creates the subTree <SubmissionBeforeClipping> to display the Submission before the clipping. used for debugging
     * @return Element
     */
    public Element createSubmission()
    {
	Element ref = document.createElement("SubmissionBeforeClipping");
	
	for(Enumeration e=videosub.segmentationBeforeClipping.elements();e.hasMoreElements();)
	    {
	

		Vector story = (Vector) e.nextElement();
		Element bound = document.createElement("Story");

		TimeBase tb = (TimeBase) story.get(0);
		TimeBase te = (TimeBase) story.get(1);
		bound.setAttribute("bTimeD",tb.toString());
		//bound.setAttribute("bTimeA",tb.add(videosub.getAlign()).toString());
		//bound.setAttribute("eTimeD",te.toString());
		bound.setAttribute("Type",(String)story.get(2));
		ref.appendChild(bound);
	    }
	return ref;
	
    }

    /**
     * It creates the subTree <SubmissionAfterClipping> to display the submission after the clipping. used for debugging
     * @return Element
     */
    public Element createSubmissionAfterClipping()
    {
	Element ref = document.createElement("SubmissionAfterClipping");
	
	for(Enumeration e=videosub.segmentation.elements();e.hasMoreElements();)
	    {
	

		Vector story = (Vector) e.nextElement();
		Element bound = document.createElement("Story");

		TimeBase tb = (TimeBase) story.get(0);
		TimeBase te = (TimeBase) story.get(1);
		bound.setAttribute("bTimeD",tb.toString());
		//bound.setAttribute("bTimeA",tb.add(videosub.getAlign()).toString());
		//bound.setAttribute("eTimeD",te.toString());
		bound.setAttribute("Type",(String)story.get(2));
		ref.appendChild(bound);
	    }
	return ref;
	
    }

}
