/**
 * This class contains all the information for one XML file submitted by a group.
 */
import org.w3c.dom.*;
import javax.xml.parsers.*;
import java.util.Vector;
import java.io.File;

public class Submission 
{
    private final int numberOfRuns=10;

    private Document doc;
    private Node sub;
    private Node currentRun;
    private Node currentVideo;
    private Vector listTruth;

    
    /**Constructor method, it parses the Xml file in param
     *param submitXML a XML file well formed as the storysegmentation dtd defines {@link http://www-nlpir.nist.gov/projects/tv2003/dtds/storySegmentationSubmission.dtd}
     */
    public Submission(File submitXML)
    {
	try{
	    DocumentBuilderFactory dbf=DocumentBuilderFactory.newInstance();
	    DocumentBuilder db = dbf.newDocumentBuilder();
	    doc=db.parse(submitXML);
	    sub=doc.getDocumentElement();
	}catch (Exception e) {
	    System.out.println("error");
	    e.getMessage();
	    e.printStackTrace();
	}
    }
    
    /**
     * Get the Root Node of the XML document
     *@return a Node XML object
     */
    public Node getRoot()
    {
	return this.sub;
    }

    /**it check if one specidic run has result for the classification task
     *@return boolean
     *@param run Node of the currrent run of the XML file
     */
    public boolean getTyped(Node run)
    {
	boolean ret=false;

	NamedNodeMap atts = run.getAttributes();
	Node tmp = atts.getNamedItem("typed");
	if(tmp.getNodeValue().equals("yes"))
	    ret= true;
	else if (tmp.getNodeValue().equals("no"))
	    ret= false;
	else System.out.println("Error XML not correct the typed attribute must be yes or no");
		
	return ret;
    }
    
    /** Get the sysID of one run
     *@return String ie the sysId
     *@param run Node object which the current run
     */
    public String getSysId(Node run)
    {
	NamedNodeMap atts = run.getAttributes();
	Node tmp = atts.getNamedItem("sysId");
	return tmp.getNodeValue();
    }
    /** Get the priority of one run
     *@return int ie the priority
     *@param run Node object which the current run
     */
    public int getPriority(Node run)
    {
	NamedNodeMap atts = run.getAttributes();
	Node tmp = atts.getNamedItem("priority");
	Integer runNo = new Integer(tmp.getNodeValue());
	return runNo.intValue();
    }

    /** Get the condition of one run
     *@return int ie the priority
     *@param run Node object which the current run
     */
    public int getCondition(Node run)
    {
	NamedNodeMap atts = run.getAttributes();
	Node tmp = atts.getNamedItem("condition");
	Integer runNo = new Integer(tmp.getNodeValue());
	return runNo.intValue();
    }

    /**Get the all the runs of the current XML file and add tehn as a Node Object in the returned Vector
     *@return Vector of Node object
     */
    public Vector getRun()
    {
	Vector tmpNo=new Vector();
	if(sub.getNodeType() == Node.ELEMENT_NODE)
	    {
		NodeList runs = sub.getChildNodes();
		for(int i=0;i<runs.getLength();i++)
		    {
			if(runs.item(i).getNodeType()==Node.ELEMENT_NODE && runs.item(i).getNodeName().equals("storySegmentationRunResult"))
			    {
				NamedNodeMap atts = runs.item(i).getAttributes();
				Node tmp = atts.getNamedItem("priority");
				Integer runNo = new Integer(tmp.getNodeValue());
				tmpNo.add(runs.item(i));
			    }
		    }
	    }
	if (tmpNo==null)
	    {
		System.out.println("No Runs found");
		return null;
	    }
	else 
	    return tmpNo;
    }

    /** this methos gets the the jth video Node of a specific run
     *@return Node Object whisch the result for one video
     *@param run the current run which is processed
     */
    public Node getVideo(int j, Node run)
    {
	int k=0;
	Node tmpNo=null;
	
	NodeList videoList = run.getChildNodes();
	int nbElementsInRun = videoList.getLength();
	
	for(int i=0;i<nbElementsInRun;i++)
	    {
		if(videoList.item(i).getNodeType()==Node.ELEMENT_NODE && videoList.item(i).getNodeName().equals("fileResult"))
		    {
			if(k==j)
			    {
				tmpNo= videoList.item(i);
				break;
			    }
			else
			    k++;
		    }
	    }
	//System.out.println("Here is the name of the Node returned by getVideo(i) : " + tmpNo.getNodeName());
	return tmpNo;
    }

    /** get the number of videos contained in a run
     *@return int
     *@param run the runto process
     */
    public int getNumberOfVideo(Node run)
    {
	NodeList videoList = run.getChildNodes();
	return videoList.getLength();
    }

}
