/**
 * This class inherits from StoriesSegment Class. An object from this class contains every information of one Reference file.
 *
 */

import java.util.Vector;
import java.util.*;
import java.io.*;

public class TruthData extends StoriesSegment
{
    private LineNumberReader lnr;
    private int nLine=0;
    private String nameFile;
    private int numberBoundaries;
    private TimeBase offset;
    
    /**
     * Basic constructor of the Class, call the constructor of the super Class StroriesSegment 
     *
     */
    public TruthData()
    {
	super();
	numberBoundaries=0;
    }

    /**
     * Constructor of the Class, call the constructor of the super Class StroriesSegment and load the Reference file in parameter 
     *@param src Reference file to load, this file must have the extension src_sgm 
     */
    public TruthData(File src)
    {
	super();
	try{
	    //this.align = offsets;
	    numberBoundaries=0;
	    FileInputStream fis = new FileInputStream(src);
	    InputStreamReader isr = new InputStreamReader(fis);
	    lnr = new LineNumberReader(isr);
	    load();
	    fis.close();
	} catch (Exception e) {
	    System.out.println("error");
	    e.getMessage();
	    e.printStackTrace();
	}
    }
    
    
    
    /**
     * Each Reference file has a begin recording time like 18:30:00.00. this begin recording Time is set as an offset and will be applied to all the time of the Reference.
     * This functtion returns this offset for this Reference file
     *@return offset retruned as a TimeBase object 
     */
    public TimeBase getOffset()
    {
	return this.offset;
    }
    
    /**
     * Load the Reference file. The reference file having the extension src_sgm. This method workds only for the file which has the same structure as the src_sgm ones
     */
    public void load()
    {
	boolean docNo = false;
	String type, line;
	TimeBase bTime, eTime;
	int nbInfo=0;
	
	type="";
	bTime= new TimeBase(0);
	offset= new TimeBase(0);
	eTime= new TimeBase(0);

	while(true){
	    try
		{
		    line = lnr.readLine();
		
	    
		    // detection of EOF
		    if (line == null) break;
		    
		    if (line.startsWith("<DOCNO>") && docNo==false)
			{
			    this.nameFile = getFileName(line);
			    docNo=true;
			}
		    
		    if (line.startsWith("<DOCTYPE>") )
			{
			    type = getType(line);
			}
		    
		    if (line.startsWith("<DATE_TIME>"))
			{
			    bTime = getTime(line);
			    if (nbInfo==0)
				offset = bTime;
			    nbInfo++;
			    
			}
		    if (line.startsWith("<END_TIME>"))
			{
			    eTime = getTime(line);
			    addOneStory(bTime.getDifferenceBetween(offset),eTime.getDifferenceBetween(offset),type);
			    
			    numberBoundaries++;
			}
		    nLine++;
		}catch (Exception e) {
		    System.out.println("error in loading the Reference :"+getVideoFileName());
		    e.getMessage();
		    e.printStackTrace();
		}
	}
	//System.out.println("here is the offset for this truth : "+getAlign());
    }
    
    /**
     * In order to compare the submission(Digital) and the reference (Analog) we have to use the same time scale. We decided to use the Analog time scale. That's why the offset for the Reference must be 0
     *@return offset retruned as a TimeBase object : offset=AnalogTime - DigitalTime
     */
    public TimeBase getAlign()
    {
	return new TimeBase(0);
    }

    /**
     * Implementation of the method clone from Object class
     *  
     */
    public Object clone()
    {
	TruthData result = new TruthData();
	for( Enumeration e = this.segmentation.elements(); e.hasMoreElements();)
	    {
		Vector story = (Vector) e.nextElement();
		Vector copyStory = (Vector)story.clone();
		result.segmentation.add(copyStory);		
	    }
	//result.clippingPoints=this.clippingPoints;
	result.offsetAnalogMinusDigital=this.offsetAnalogMinusDigital;
	result.nameFile = this.nameFile;
	result.offset=this.offset;
	return result;

    }

    /**
     * This method apply the clipping points to the current Video. But it doesn't use the clipping points of the txt file
     * @return Vector of Story clipped
     * <pre>
     *
     * if ($offset >0)
     *  #MPEG starts later the ground truth, we have to remove initial stories from the ground Truth
     *  # remove story in interval [0.0 -- offset]
     *
     *   GGGGGGGG
     *       MMMMMMMMM
     *   DDDD
     *  (G is ground truth, M is mpeg, D is delete)
     *
     * if ($length + $offset < 1800) 
     *   # MPEG ends before the gound truth, we have to remove some
     *   final stories from the grounf Truth 
     *   # remove story in interval [ (1800 -$offset) -- $length]
     *
     *  GGGGGGGGGGGGG
     *  MMMMMMMM
     *          DDDDD
     * </pre>
     */
 
    public Vector applyClippingPoints()
    {
	//System.out.println("apply clipping Truth");
	Vector line= new Vector();
	
	Vector clippedTmp = new Vector();
	
	//line = searchClippingPoints(); //get the clipping points for the current truth
	this.segmentationBeforeClipping = this.segmentation;
	
	if (this.treated)
	  {
	//TimeBase start = (TimeBase) line.get(1);
	//TimeBase end = (TimeBase) line.get(2);
		
		this.lengthVideo=getMPEG1Length();

		
		if(offsetAnalogMinusDigital.getint() > 0)
		    {
			Vector newSegmentation= new Vector();
			for (int i=0;i<getNumberOfStories();i++)
			    {
				if (getBeginTimeStory(i) > offsetAnalogMinusDigital.getint() && getEndTimeStory(i) > offsetAnalogMinusDigital.getint() )
				    {
					newSegmentation.add(segmentation.get(i));
				    }
				    
				else if(getBeginTimeStory(i) >= 0  && getEndTimeStory(i) <= offsetAnalogMinusDigital.getint() )
				    {
					Vector storyClippedVector = new Vector();
					
					TimeBase tclippedBegin = new TimeBase(getBeginTimeStory(i));
					TimeBase tclippedEnd = new TimeBase(getEndTimeStory(i));
					
					storyClippedVector.add(tclippedBegin);
					storyClippedVector.add(tclippedEnd);
					storyClippedVector.add(getType(i));
					
					clippedTmp.add(storyClippedVector);
				    }

				else if (getBeginTimeStory(i) < offsetAnalogMinusDigital.getint() &&  getEndTimeStory(i) > offsetAnalogMinusDigital.getint() )
				    {
					Vector oneStory = new Vector();
					Vector storyClippedVector = new Vector();
					
					oneStory.add(new TimeBase(offsetAnalogMinusDigital.getint()));
					oneStory.add(new TimeBase(getEndTimeStory(i)));
					oneStory.add(getType(i));
					newSegmentation.add(oneStory);
					
					TimeBase tclipped = new TimeBase(offsetAnalogMinusDigital.getint());
					storyClippedVector.add(new TimeBase(getBeginTimeStory(i)));
					storyClippedVector.add(tclipped);
					storyClippedVector.add(getType(i));
					clippedTmp.add(storyClippedVector);
				    
				    }
				else if (getBeginTimeStory(i) == offsetAnalogMinusDigital.getint() && getEndTimeStory(i) > offsetAnalogMinusDigital.getint())
				    {
					newSegmentation.add(segmentation.get(i));
					
					Vector storyClippedVector = new Vector();
					storyClippedVector.add(new TimeBase(getBeginTimeStory(i)));
					storyClippedVector.add(new TimeBase(getBeginTimeStory(i)));
					storyClippedVector.add(getType(i));
					clippedTmp.add(storyClippedVector);
				    }

			    }
			this.segmentation = newSegmentation;
		    }
		

		if( this.lengthVideo.getint() + offsetAnalogMinusDigital.getint() < 180000)
		    {
			Vector newSegmentation= new Vector();
			for (int i=0;i<getNumberOfStories();i++)
			    {
				if (getBeginTimeStory(i) < this.lengthVideo.getint()+ offsetAnalogMinusDigital.getint() && getEndTimeStory(i) < this.lengthVideo.getint()+ offsetAnalogMinusDigital.getint() )
				    {
					newSegmentation.add(segmentation.get(i));
				    }
				    
				else if(getBeginTimeStory(i) >= this.lengthVideo.getint()+ offsetAnalogMinusDigital.getint()  &&  getEndTimeStory(i) <= 180000 )
				    {
					Vector storyClippedVector = new Vector();
					TimeBase tclippedBegin = new TimeBase(getBeginTimeStory(i));
					TimeBase tclippedEnd = new TimeBase(getEndTimeStory(i));
					
					storyClippedVector.add(tclippedBegin);
					storyClippedVector.add(tclippedEnd);
					storyClippedVector.add(getType(i));
					
					clippedTmp.add(storyClippedVector);
				    }

				else if (getBeginTimeStory(i) < this.lengthVideo.getint()+ offsetAnalogMinusDigital.getint() &&  getEndTimeStory(i) > this.lengthVideo.getint()+ offsetAnalogMinusDigital.getint() )
				    {
					Vector oneStory = new Vector();
					Vector storyClippedVector = new Vector();
					
					oneStory.add(new TimeBase(getBeginTimeStory(i)));
					oneStory.add(new TimeBase(this.lengthVideo.getint()+ offsetAnalogMinusDigital.getint() ));
					oneStory.add(getType(i));
					newSegmentation.add(oneStory);
					
					TimeBase tclipped = new TimeBase(getEndTimeStory(i));
					storyClippedVector.add(new TimeBase(this.lengthVideo.getint()+ offsetAnalogMinusDigital.getint()));
					storyClippedVector.add(tclipped);
					storyClippedVector.add(getType(i));
					clippedTmp.add(storyClippedVector);
				    
				    }
				

			    }
			this.segmentation = newSegmentation;
		    }
		removeFirstEnd(clippedTmp);
		}
	//System.out.println("clipping Truth finished");
 	return clippedTmp;
    }

    public void removeFirstEnd(Vector clipped)
    {
	Vector clippedTmp=clipped;
	Vector newSegmentation= new Vector();
		//drop the boundary equal to 0 and equal to the length of the video.. because thare ar not measured
		for (int i=0;i<getNumberOfStories();i++)
		    {
			if(getBeginTimeStory(i)==0)
			    {
				Vector oneStory = new Vector();
				Vector storyClippedVector = new Vector();
					
				oneStory.add(new TimeBase(getBeginTimeStory(i)));
				oneStory.add(new TimeBase(getEndTimeStory(i)));
				oneStory.add(getType(i));
				newSegmentation.add(oneStory);
					
				TimeBase tclipped = new TimeBase(getBeginTimeStory(i));
				storyClippedVector.add(tclipped);
				storyClippedVector.add(tclipped);
				storyClippedVector.add(getType(i));
				clippedTmp.add(storyClippedVector);
			    }
			else if(getEndTimeStory(i)==180000)
			    {
				Vector oneStory = new Vector();
				Vector storyClippedVector = new Vector();
					
				oneStory.add(new TimeBase(getBeginTimeStory(i)));
				oneStory.add(new TimeBase(getEndTimeStory(i)));
				oneStory.add(getType(i));
				newSegmentation.add(oneStory);
					
				TimeBase tclipped = new TimeBase(getEndTimeStory(i));
				storyClippedVector.add(tclipped);
				storyClippedVector.add(tclipped);
				storyClippedVector.add(getType(i));
				clippedTmp.add(storyClippedVector);
			    }
			else
			    {
				Vector oneStory = new Vector();
				Vector storyClippedVector = new Vector();
					
				oneStory.add(new TimeBase(getBeginTimeStory(i)));
				oneStory.add(new TimeBase(getEndTimeStory(i)));
				oneStory.add(getType(i));
				newSegmentation.add(oneStory);

			    }

		    }
		this.segmentation = newSegmentation;
    }
    
    
    /**
     * Returns the number of boundaries evaluated of the current StoriesSegment. Ie those inside the clipping points expect those ones
     * @return int
     */
 
      public int getNumberOfEvaluatedBoundaries()
    {
	
	int result=0;
	for(int i=1;i<getNumberOfBoundaries()+1;i++)
	    {
		if(
		   !( (offsetAnalogMinusDigital.getint()>0 &&  getTimeOfBoundary(i) == offsetAnalogMinusDigital.getint()) )
		   && getTimeOfBoundary(i) !=0
		   && getTimeOfBoundary(i) != 180000
		   && !(( this.lengthVideo.getint())+offsetAnalogMinusDigital.getint() < 180000 &&  getTimeOfBoundary(i) == this.lengthVideo.getint()+ offsetAnalogMinusDigital.getint())
		   
		   )
		    result++;

	    }
	return result;
    }
	
     /**
     * Returns the name of the video associated to the current truth from the src_sgm file
     * @return String name of the file ex : 19980405_ABC.mpg
     * @param line line in the src_sgm file this method analyse
     */
    public String getFileName(String line)
    {
	String date = line.substring(11,19);
	String from = line.substring(8,11);
	return date.concat("_").concat(from).concat("a.mpg");
    }

    public void addEndTime()
    {
	//System.out.println("AddEndTime called Truth ");	
    }
    

    /**
     * Returns the name of the video associated to the current truth 
     * @return String name of the file ex : 19980405_ABC.mpg
     */    
    public String getVideoFileName()
    {
	return this.nameFile;
    }

     /**
     * Returns the type of the story from a line of a src_sgm file
     * @return String type of the stories ex : news or misc for TRECVID2003
     * @param line String line  in the src_sgm file this method analyse
     */
    public String getType(String line)
    {
	String tmp= line.substring(9,line.length()-10).trim();
	if (tmp.startsWith("MISC"))
		{
		    return "misc";
		}
	    else
		{
		    return "news";
    		}
    }

    /**
     * Returns the time of line of a src_sgm file
     * @return TimeBase object
     * @param String line from the src_sgm file. ex : <END_TIME> 06/30/1998 21:39:46.78 </END_TIME>
     */
    public TimeBase getTime(String line)
    {
	String[] table = line.split(" ");
	TimeBase tb = new TimeBase(table[2],"src_sgm");
	//System.out.println("here is the time return by Time Base : "+ tb.getint());
	return tb;
    }

}
