/**
 * This class inherits from StoriesSegment Class. An object from this class contains every information of a Video resulst from on specific Run submited by one group.
 *
 */
import java.util.Vector;
import java.sql.Time;
import org.w3c.dom.*;
import javax.xml.parsers.*;
import java.io.*;

public class VideoSubmission extends StoriesSegment
{
    final String datafile="../Data/"; //this variable is also defined in StorySegmentationEvaluation Class
    private Node videoNode;
    private TruthData truth;
    private int numberBoundariesSub, numberStories; //should be in the mother class
    private boolean typed;
    private Vector align;

    /**
     * Constructor of the class.
     * @param typed boolean says if the run whre that video is coming from is typed or not
     * @param video XML Node associated to the current video Results
     * @param clippingPoints Vector containing all the clipping points
     * @param offsets Vector containing all the offsets
     */
    public VideoSubmission(boolean typed, Node video, Vector offsets)
    {
	super();
	this.align = offsets;
	this.videoNode = video;
	//this.clippingPoints=clippingPoints;
	this.typed=typed;
	if(video == null)
	    System.out.println("Each run must contain the same number of video ad the truth directory contains");
		
	load();
	addEndTime();
	//this.treated=true;
	this.offsetAnalogMinusDigital = getAlign();
	clippedSub = applyClippingPoints();
    }
    

    /**
     * Constructor of the class.
     * @param typed boolean says if the run whre that video is coming from is typed or not
     * @param video XML Node associated to the current video Results
     */    
    public VideoSubmission(boolean typed, Node video)
    {
	super();
	this.videoNode = video;
	
    }

    /**
     * This method associates a TruthData object to the current Video results
     * @return void
     * @param truth A truthData object
     */
    public void setTruthData(TruthData truth)
    {
	//truth.clippingPoints=this.clippingPoints;
	truth.offsetAnalogMinusDigital = getAlign();
	clippedRef = truth.applyClippingPoints();
	this.truth=truth;
    }
    
    /**
     * This method apply the clipping points to the current Video. But it doesn't use the clipping points of the txt file
     * @return Vector of Story clipped
     * <pre>
     *
     * if ($offset <0)
     *  #MPEG starts before the ground truth, we have to remove initial stories from the submission
     *  # remove story in interval [0.0 -- offset]
     *
     *       GGGGGGGG
     *   MMMMMMMMM
     *   DDDD
     *  (G is ground truth, M is mpeg, D is delete)
     *
     * if ( 1800-$offset < $length) 
     *   # ground truth ends before the end of the video, we have to remove some
     *   final stories from
     *   # submission
     *   # remove story in interval [ (1800 -$offset) -- $length]
     *
     *  GGGGGGGG
     *  MMMMMMMMMMMMM
     *          DDDDD
     * </pre>
     */

    public Vector applyClippingPoints()
    {
	//System.out.println("apply clipping Submission for "+getVideoFileName());
	
	Vector line= new Vector();
	
	Vector clippedTmp = new Vector();
	
	//line = searchClippingPoints(); //get the clipping points for the current truth
	this.segmentationBeforeClipping = this.segmentation;

	if (this.treated)
	  {
	//TimeBase start = (TimeBase) line.get(1);
	//TimeBase end = (TimeBase) line.get(2);
		
		this.lengthVideo=getMPEG1Length();
		
		if(offsetAnalogMinusDigital.getint() <0)
		    {
			Vector newSegmentation= new Vector();
			for (int i=0;i<getNumberOfStories();i++)
			    {
				
				if (getBeginTimeStory(i) > -offsetAnalogMinusDigital.getint() && getEndTimeStory(i) > -offsetAnalogMinusDigital.getint() )
				    {
					newSegmentation.add(segmentation.get(i));
				    }
				    
				else if(getBeginTimeStory(i) >= 0  &&   getEndTimeStory(i) <= -offsetAnalogMinusDigital.getint() )
				    {
					Vector storyClippedVector = new Vector();
					
					TimeBase tclippedBegin = new TimeBase(getBeginTimeStory(i));
					TimeBase tclippedEnd = new TimeBase(getEndTimeStory(i));
					
					storyClippedVector.add(tclippedBegin);
					storyClippedVector.add(tclippedEnd);
					storyClippedVector.add(getType(i));
					
					clippedTmp.add(storyClippedVector);
				    }

				else if (getBeginTimeStory(i) < -offsetAnalogMinusDigital.getint() &&  getEndTimeStory(i) > -offsetAnalogMinusDigital.getint() )
				    {
					Vector oneStory = new Vector();
					Vector storyClippedVector = new Vector();
					
					oneStory.add(new TimeBase(-offsetAnalogMinusDigital.getint()));
					oneStory.add(new TimeBase(getEndTimeStory(i)));
					oneStory.add(getType(i));
					newSegmentation.add(oneStory);
					
					TimeBase tclipped = new TimeBase(-offsetAnalogMinusDigital.getint());
					storyClippedVector.add(new TimeBase(getBeginTimeStory(i)));
					storyClippedVector.add(tclipped);
					storyClippedVector.add(getType(i));
					clippedTmp.add(storyClippedVector);
				    }
				else if (getBeginTimeStory(i) == -offsetAnalogMinusDigital.getint() && getEndTimeStory(i) > -offsetAnalogMinusDigital.getint())
				    {
					newSegmentation.add(segmentation.get(i));
					
					Vector storyClippedVector = new Vector();
					storyClippedVector.add(new TimeBase(getBeginTimeStory(i)));
					storyClippedVector.add(new TimeBase(getBeginTimeStory(i)));
					storyClippedVector.add(getType(i));
					clippedTmp.add(storyClippedVector);
				    }
			    }
			this.segmentation = newSegmentation;
		    }

		if( 180000 - offsetAnalogMinusDigital.getint() < this.lengthVideo.getint() )
		    {
			Vector newSegmentation= new Vector();
			for (int i=0;i<getNumberOfStories();i++)
			    {
			
				if (getBeginTimeStory(i) < (180000 - offsetAnalogMinusDigital.getint()) && getEndTimeStory(i) < (180000 - offsetAnalogMinusDigital.getint()) )
				    {
					newSegmentation.add(segmentation.get(i));
				    }
				    
				else if(getBeginTimeStory(i) >= 180000 - offsetAnalogMinusDigital.getint()  &&  getEndTimeStory(i) <= this.lengthVideo.getint() )
				    {
					Vector storyClippedVector = new Vector();
					
					TimeBase tclippedBegin = new TimeBase(getBeginTimeStory(i));
					TimeBase tclippedEnd = new TimeBase(getEndTimeStory(i));
					
					storyClippedVector.add(tclippedBegin);
					storyClippedVector.add(tclippedEnd);
					storyClippedVector.add(getType(i));
					
					clippedTmp.add(storyClippedVector);
				    }

				else if (getBeginTimeStory(i) < 180000 - offsetAnalogMinusDigital.getint() &&  getEndTimeStory(i) > 180000 - offsetAnalogMinusDigital.getint() )
				    {
					Vector oneStory = new Vector();
					Vector storyClippedVector = new Vector();
					
					oneStory.add(new TimeBase(getBeginTimeStory(i)));
					oneStory.add(new TimeBase(180000 - offsetAnalogMinusDigital.getint()));
					oneStory.add(getType(i));
					newSegmentation.add(oneStory);
					
					TimeBase tclipped = new TimeBase(getEndTimeStory(i));
					storyClippedVector.add(new TimeBase(180000 - offsetAnalogMinusDigital.getint()));
					storyClippedVector.add(tclipped);
					storyClippedVector.add(getType(i));
					clippedTmp.add(storyClippedVector);
				    }
		
			    }
			this.segmentation = newSegmentation;
		    }
		removeFirstEnd(clippedTmp);
		
		}
 	return clippedTmp;
    }
    
    public void removeFirstEnd(Vector clipped)
    {
	Vector clippedTmp=clipped;
	Vector newSegmentation= new Vector();
		//drop the boundary equal to 0 and equal to the length of the video.. because thare ar not measured
		for (int i=0;i<getNumberOfStories();i++)
		    {
			if(getBeginTimeStory(i)==0)
			    {
				Vector oneStory = new Vector();
				Vector storyClippedVector = new Vector();
					
				oneStory.add(new TimeBase(getBeginTimeStory(i)));
				oneStory.add(new TimeBase(getEndTimeStory(i)));
				oneStory.add(getType(i));
				newSegmentation.add(oneStory);
					
				TimeBase tclipped = new TimeBase(getBeginTimeStory(i));
				storyClippedVector.add(tclipped);
				storyClippedVector.add(tclipped);
				storyClippedVector.add(getType(i));
				clippedTmp.add(storyClippedVector);
			    }
			else if(getEndTimeStory(i)==this.lengthVideo.getint())
			    {
				Vector oneStory = new Vector();
				Vector storyClippedVector = new Vector();
					
				oneStory.add(new TimeBase(getBeginTimeStory(i)));
				oneStory.add(new TimeBase(getEndTimeStory(i)));
				oneStory.add(getType(i));
				newSegmentation.add(oneStory);
					
				TimeBase tclipped = new TimeBase(getEndTimeStory(i));
				storyClippedVector.add(tclipped);
				storyClippedVector.add(tclipped);
				storyClippedVector.add(getType(i));
				clippedTmp.add(storyClippedVector);
			    }
			else
			    {
				Vector oneStory = new Vector();
				Vector storyClippedVector = new Vector();
					
				oneStory.add(new TimeBase(getBeginTimeStory(i)));
				oneStory.add(new TimeBase(getEndTimeStory(i)));
				oneStory.add(getType(i));
				newSegmentation.add(oneStory);

			    }

		    }
		this.segmentation = newSegmentation;
    }


    /**
     * Add the endTime of the last story of the segmentation ie the length of the Mpeg1
     * @return void
     *
     */
    public void addEndTime()
    {
	//System.out.println("AddEndTime called Sub ");
	Vector tmp = (Vector) this.segmentation.get(getNumberOfStories()-1);
	TimeBase newTime = getMPEG1Length();
	tmp.setElementAt(newTime,1);
	this.segmentation.setElementAt(tmp,getNumberOfStories()-1);
    }

    /**
     * This method let us know il the result for this videos contains classification information
     * @return boolean true if the run is typed and false if it is not.
     */
    public boolean getTyped()
    {
	return typed;
    }
    
    /**
     * Returns the name of the video associated to the current truth 
     * @return String name of the file ex : 19980405_ABC.mpg
     */ 
    public String getVideoFileName()
    {
	NamedNodeMap atts = videoNode.getAttributes();
	Node tmp = atts.getNamedItem("src");
	return tmp.getNodeValue();
    }
    
    /**
     * Load the Video results from the XML file and fill the attributes of VideoSubmission
     * @return void
     */
    public void load()
    {
	//System.out.println("load Video ");
	
	String beginTime, type;
	TimeBase nextTime=null;
	
	NodeList boundaries = this.videoNode.getChildNodes();
	int numberBoundariesSub = 0;
	
	for(int i=boundaries.getLength()-1; i>-1;i--)
	    {
		
		if(boundaries.item(i).getNodeType()==Node.ELEMENT_NODE && boundaries.item(i).getNodeName().equals("story"))
		    {
			NamedNodeMap atts = boundaries.item(i).getAttributes();
			Node tmp = atts.getNamedItem("bTime");
			beginTime = tmp.getNodeValue();
			if (getTyped())
			    {
				Node tmp1 = atts.getNamedItem("type");
				type=tmp1.getNodeValue();
			    }
			else type="";
					
			    if (i==0)
			    {
				//System.out.println("btime : " +" EndTime : " + nextTime+ " Type : "+type);
				addOneStory(new TimeBase(0), nextTime, type);
			    }
			else
			    {
				TimeBase bTime = new TimeBase(beginTime,"submission");
				//System.out.println("btime : " + bTime +" EndTime : " + nextTime+ " Type : "+type); 
				
				addOneStory(bTime,nextTime ,type);
				nextTime=bTime;
			    }
					    			
			numberBoundariesSub++;
		    }
	    }
	
	this.segmentation = invertStoriesSegment(segmentation);
	
    }
    

    /**
     * returns Number Of detected Boundaries for this video. It loops on the boundaries of the video and search if there one which match in the associated truth +-fuzzyfactor.
     * Every times are convected to analog times and are compared.
     * @return int nb of detected boundaries
     * @param fuzzyfactor int 
     */

    public int getNbDetectedBoundaries(double fuzzyFactor)
    {
	//System.out.println("getNbDetectedBoundaries() function is called");
	
	Vector line= new Vector();
	//line = searchClippingPoints();
	//TimeBase start = (TimeBase) line.get(1);
	//TimeBase end = (TimeBase) line.get(2);
	
	//System.out.println("clippingstart : "+start.getint()+" clippingstop : "+end.getint());
	int result=0;

	if(this.truth == null)
	    System.out.println("The truth dada was not set for this video file");
	else 
	    {
	
		//System.out.println("size of segmentation : " + segmentation.size());
		//System.out.println("return by getNb of boundaries sub : " +getNumberOfBoundaries() );
		//System.out.println("Nb of Boundaries in Truth : " + truth.getNumberBoundaries());
		for(int i=1;i<getNumberOfBoundaries()+1;i++)
		    {
			//System.out.println("Time of boundary : "+getTimeOfBoundary(i)+ " for i :" + i+ " number of boundaries : "+ getNumberOfBoundaries());
			
			boolean boundarySubIsclippingPoint = (getTimeOfBoundary(i)==(180000 - offsetAnalogMinusDigital.getint()) && (180000 - offsetAnalogMinusDigital.getint() <this.lengthVideo.getint())) || (getTimeOfBoundary(i)== -offsetAnalogMinusDigital.getint() && offsetAnalogMinusDigital.getint() < 0);
			boolean found=false;
			
			// the boundarie equal to the clipping points musnt't be evaluated thus the last boundary equal to the length of the MPEG
			if(!boundarySubIsclippingPoint && getTimeOfBoundary(i) != this.lengthVideo.getint() && getTimeOfBoundary(i) !=0)
			    {
			
				for(int j=1;j<truth.getNumberOfBoundaries()+1;j++)
				    {
					
					//System.out.println("time of boundary j : "+getTimeOfBoundary(j));
					boolean boundaryRefIsClippingPoint = truth.getTimeOfBoundary(j)==offsetAnalogMinusDigital.getint() && offsetAnalogMinusDigital.getint() >0 || truth.getTimeOfBoundary(j)== offsetAnalogMinusDigital.getint()+ this.lengthVideo.getint() && this.lengthVideo.getint() +offsetAnalogMinusDigital.getint() <180000 ;
					boolean a = (getTimeOfBoundary(i)+getAlign().getint()>= truth.getTimeOfBoundary(j)- fuzzyFactor*100) && (getTimeOfBoundary(i)+getAlign().getint() <= truth.getTimeOfBoundary(j)+ fuzzyFactor*100);
					// To score the sbmitted boundaries mustn't match with the clipping points and the end of the truth neither.
					if(a && !boundaryRefIsClippingPoint && truth.getTimeOfBoundary(j) != 180000 && truth.getTimeOfBoundary(j)!=0)
					    {
						result++;
						//if(i!=getNumberOfBoundaries())
						//  {
							Vector tmp = new Vector();
							TimeBase time = new TimeBase(getTimeOfBoundary(i));
							tmp.add(time.toString());
							tmp.add(getType(i-1));
							match.add(tmp);
							found=true;
							//  }
						break;
					    }
					else 
					    {
						// This boudary of the submission is not in the reference
						found=false;
					    }
				    }
			    }
			if(i!=getNumberOfBoundaries() && found==false && !boundarySubIsclippingPoint && getTimeOfBoundary(i) != this.lengthVideo.getint() && getTimeOfBoundary(i) !=0)
			    {
				Vector tmp = new Vector();
				TimeBase time = new TimeBase(getTimeOfBoundary(i));
				tmp.add(time.toString());
				tmp.add(getType(i-1));
				insertion.add(tmp);
			    }
		    }
	    }
	//System.out.println("Number of boundaries detectected : "+result);
	//System.out.println("Nb of Boundaries in Truth : " + truth.getNumberOfEvaluatedBoundaries());
	//System.out.println("return by getNb of boundaries sub : " +getNumberOfEvaluatedBoundaries() );
	return result;
    }


    
    /**
     * returns Number Of detected Reference Boundaries for this video. It loops on the boundaries of the TruthData object associated and search if there one which match in the current object +-fuzzyfactor
     * Every times are convected to analog times and are compared.
     * @return int nb of detected referenceboundaries
     * @param fuzzyfactor int for TRECVID2003 fuzzyfactor =5
     */
    public int getNumbersOfReferenceBoundariesDetectedOneFile(double fuzzyFactor)
    {
	//System.out.println("get number one file called");
	int result=0;
	
	Vector line= new Vector();
	//line = searchClippingPoints();
	//TimeBase start = (TimeBase) line.get(1);
	//TimeBase end = (TimeBase) line.get(2);
	
	if(this.truth == null)
	    System.out.println("The truth dada was not set for this video file");
	else 
	    {
		//truth.displaySegment();
		for(int i=1;i<truth.getNumberOfBoundaries()+1;i++)
		    {
			boolean boundaryRefIsClippingPoint = truth.getTimeOfBoundary(i)==offsetAnalogMinusDigital.getint() && offsetAnalogMinusDigital.getint() >0 
			    || truth.getTimeOfBoundary(i)== offsetAnalogMinusDigital.getint()+ this.lengthVideo.getint() && (this.lengthVideo.getint()+offsetAnalogMinusDigital.getint()) <180000 ;
			boolean found=false;

			//System.out.println("Time of boundary : "+getTimeOfBoundary(i)+ " for i :" + i+ " number of boundaries : "+ getNumberOfBoundaries());
			//Evaluate Only the boundaries which are not clipping points
			if(!boundaryRefIsClippingPoint && truth.getTimeOfBoundary(i) != 180000 && truth.getTimeOfBoundary(i) != 0)
			    {
				for(int j=1;j<getNumberOfBoundaries()+1;j++)
				    {
					boolean boundarySubIsClippingPoint = (getTimeOfBoundary(j)==(180000 - offsetAnalogMinusDigital.getint()) && (180000 - offsetAnalogMinusDigital.getint() <this.lengthVideo.getint())) || (getTimeOfBoundary(j)== -offsetAnalogMinusDigital.getint() && offsetAnalogMinusDigital.getint() < 0);	
					boolean a = (truth.getTimeOfBoundary(i)>= getTimeOfBoundary(j)+getAlign().getint()- fuzzyFactor*100) && (truth.getTimeOfBoundary(i) <= getTimeOfBoundary(j)+getAlign().getint()+ fuzzyFactor*100);
					if(a && !boundarySubIsClippingPoint && getTimeOfBoundary(j)!=0  && getTimeOfBoundary(j)!= this.lengthVideo.getint() && getTimeOfBoundary(j)!=0)
					    {
						result++;
						found=true;
						break;
					    }
					
				    }
			    }
				
			if(i!=truth.getNumberOfBoundaries() && found==false && !boundaryRefIsClippingPoint && truth.getTimeOfBoundary(i)!=180000 && truth.getTimeOfBoundary(i)!=0)
			    {
				// This boundary of the reference is not in the submission
				Vector tmp = new Vector();
				TimeBase time = new TimeBase(truth.getTimeOfBoundary(i));
				tmp.add(time);
				tmp.add(truth.getType(i-1));
				deletion.add(tmp);
			    }
		    }
	    }
	//System.out.println("Number of boundaries detectected     : "+result);
	//System.out.println("Nb of Boundaries in Truth to Evaluate: " + truth.getNumberOfEvaluatedBoundaries());
	//System.out.println("Nb of boundaries sub to Evaluate     : " +getNumberOfEvaluatedBoundaries() );
	
	return result;
    }

    /**
     * Returns the number of boundaries evaluated of the current StoriesSegment. Ie those inside the clipping points expect those ones
     * @return int
     */
    public int getNumberOfEvaluatedBoundaries()
    {
	int result=0;
	for(int i=1;i<getNumberOfBoundaries()+1;i++)
	    {
		if(
		   !( (offsetAnalogMinusDigital.getint()<0 && getTimeOfBoundary(i)== -offsetAnalogMinusDigital.getint()) )
		   && getTimeOfBoundary(i) !=0
		   && getTimeOfBoundary(i) != this.lengthVideo.getint()
		   &&  !( (180000-offsetAnalogMinusDigital.getint())<this.lengthVideo.getint() && getTimeOfBoundary(i)==(180000-offsetAnalogMinusDigital.getint()) )
		   )
		    result++;

	    }
	return result;
    }


    /**
     * This method returns the total Time of a specific story type. This method calls the Fusion Class
     * @return int total time
     * @param type Type of the story, for TRECVID2003 we need the information on "news"
     */
    public int getTotalTimeIdentifiedOf(String type)
    {
	//System.out.println("getTotalTimeID called");
	int result=0;  

	if(getTyped())
	    {
		Fusion melt = new Fusion(this, this.truth);
		result = melt.getLengthOfCommunStory(type).getint();
		Vector[] resultClassiDistribution = melt.getOutputDistribution();
		
		this.matchType = resultClassiDistribution[0];
		this.nomatchType =  resultClassiDistribution[1];
		this.missingType =resultClassiDistribution[2];	   
	    }
	return result;
    }

    /**
     * Returns the offset we have to add to the submission time(Digital) to get the Analog Time. Thus we could compare the Ref and the submission on the same scale : the analog scale time.
     *@return offset retruned as a TimeBase object : offset=AnalogTime - DigitalTime
     */
    public TimeBase getAlign()
    {
	TimeBase off= new TimeBase(0);
	boolean found=false;
	
	for(int i=0;i<align.size();i++)
	    {
		Vector element = (Vector) align.get(i);
		String namefile = (String) element.get(0);
		if(namefile.equals(getVideoFileName()))
		    {
			off = (TimeBase) element.get(1);
			found =true;
			break;
		    }
	    }
	if(!found)
	    {
		System.out.println("offset not found for the video : "+getVideoFileName()+".This video won't be treated");
		//if there are no clippingpoints for this video, it mustn't be treated
		this.treated = false;
		off=null;
	    }
	return off;

    }

}
